import base64
from typing import Callable, Dict, Optional, Union

from pydantic import BaseModel, field_serializer

from cozepy.request import Requester
from cozepy.util import dump_exclude_none, remove_url_trailing_slash
from cozepy.websockets.ws import (
    AsyncWebsocketsBaseClient,
    AsyncWebsocketsBaseEventHandler,
    InputAudio,
    WebsocketsBaseClient,
    WebsocketsBaseEventHandler,
    WebsocketsEvent,
    WebsocketsEventFactory,
    WebsocketsEventType,
)


# req
class TranscriptionsUpdateEvent(WebsocketsEvent):
    """更新语音识别配置

    更新语音识别配置。若更新成功，会收到 transcriptions.updated 的下行事件，否则，会收到 error 下行事件。
    docs: https://www.coze.cn/open/docs/developer_guides/asr_event#a7ca67ab
    """

    class Data(BaseModel):
        # 输入音频格式。
        input_audio: Optional[InputAudio] = None

    event_type: WebsocketsEventType = WebsocketsEventType.TRANSCRIPTIONS_UPDATE
    data: Data


# req
class InputAudioBufferAppendEvent(WebsocketsEvent):
    """流式上传音频片段

    流式向服务端提交音频的片段。
    docs: https://www.coze.cn/open/docs/developer_guides/asr_event#9ef6e6ca
    """

    class Data(BaseModel):
        # 音频片段。(API 返回的是base64编码的音频片段, SDK 已经自动解码为 bytes)
        delta: bytes

        @field_serializer("delta")
        def serialize_delta(self, delta: bytes, _info):
            return base64.b64encode(delta)

    event_type: WebsocketsEventType = WebsocketsEventType.INPUT_AUDIO_BUFFER_APPEND
    data: Data

    def _dump_without_delta(self):
        return dump_exclude_none(
            {
                "id": self.id,
                "type": self.event_type.value,
                "detail": self.detail,
                "data": {
                    "delta": f"<length: {len(self.data.delta) if self.data and self.data.delta else 0}>",
                },
            }
        )


# req
class InputAudioBufferCompleteEvent(WebsocketsEvent):
    """提交音频

    客户端发送 input_audio_buffer.complete 事件来告诉服务端提交音频缓冲区的数据。服务端提交成功后会返回 input_audio_buffer.completed 事件。
    docs: https://www.coze.cn/open/docs/developer_guides/asr_event#f5d76c87
    """

    event_type: WebsocketsEventType = WebsocketsEventType.INPUT_AUDIO_BUFFER_COMPLETE


# req
class InputAudioBufferClearEvent(WebsocketsEvent):
    """清除缓冲区音频

    客户端发送 input_audio_buffer.clear 事件来告诉服务端清除缓冲区的音频数据。服务端清除完后将返回 input_audio_buffer.cleared 事件。
    docs: https://www.coze.cn/open/docs/developer_guides/asr_event#e98db543
    """

    event_type: WebsocketsEventType = WebsocketsEventType.INPUT_AUDIO_BUFFER_CLEAR


# resp
class TranscriptionsCreatedEvent(WebsocketsEvent):
    """语音识别连接成功

    语音识别连接成功后，返回此事件。
    docs: https://www.coze.cn/open/docs/developer_guides/asr_event#06d772a3
    """

    event_type: WebsocketsEventType = WebsocketsEventType.TRANSCRIPTIONS_CREATED


# resp
class TranscriptionsUpdatedEvent(WebsocketsEvent):
    """配置更新成功

    配置更新成功后，会返回最新的配置。
    docs: https://www.coze.cn/open/docs/developer_guides/asr_event#3f842df1
    """

    class Data(BaseModel):
        # 输入音频格式。
        input_audio: Optional[InputAudio] = None

    event_type: WebsocketsEventType = WebsocketsEventType.TRANSCRIPTIONS_UPDATED
    data: Data


# resp
class InputAudioBufferCompletedEvent(WebsocketsEvent):
    """音频提交完成

    客户端发送 input_audio_buffer.complete 事件来告诉服务端提交音频缓冲区的数据。服务端提交成功后会返回 input_audio_buffer.completed 事件。
    docs: https://www.coze.cn/open/docs/developer_guides/asr_event#8d747148
    """

    event_type: WebsocketsEventType = WebsocketsEventType.INPUT_AUDIO_BUFFER_COMPLETED


# resp
class InputAudioBufferClearedEvent(WebsocketsEvent):
    """音频清除成功

    客户端发送 input_audio_buffer.clear 事件来告诉服务端清除音频缓冲区的数据。服务端清除完后将返回 input_audio_buffer.cleared 事件。
    docs: https://www.coze.cn/open/docs/developer_guides/asr_event#8211875b
    """

    event_type: WebsocketsEventType = WebsocketsEventType.INPUT_AUDIO_BUFFER_CLEARED


# resp
class TranscriptionsMessageUpdateEvent(WebsocketsEvent):
    """识别出文字

    语音识别出文字后，返回此事件，每次都返回全量的识别出来的文字。
    docs: https://www.coze.cn/open/docs/developer_guides/asr_event#772e6d2d
    """

    class Data(BaseModel):
        # 识别出的文字。
        content: str

    event_type: WebsocketsEventType = WebsocketsEventType.TRANSCRIPTIONS_MESSAGE_UPDATE
    data: Data


# resp
class TranscriptionsMessageCompletedEvent(WebsocketsEvent):
    """识别完成

    语音识别完成后，返回此事件。
    docs: https://www.coze.cn/open/docs/developer_guides/asr_event#0c36158c
    """

    event_type: WebsocketsEventType = WebsocketsEventType.TRANSCRIPTIONS_MESSAGE_COMPLETED


_audio_transcriptions_event_factory = WebsocketsEventFactory(
    {
        WebsocketsEventType.TRANSCRIPTIONS_CREATED.value: TranscriptionsCreatedEvent,
        WebsocketsEventType.TRANSCRIPTIONS_UPDATED.value: TranscriptionsUpdatedEvent,
        WebsocketsEventType.INPUT_AUDIO_BUFFER_COMPLETED.value: InputAudioBufferCompletedEvent,
        WebsocketsEventType.INPUT_AUDIO_BUFFER_CLEARED.value: InputAudioBufferClearedEvent,
        WebsocketsEventType.TRANSCRIPTIONS_MESSAGE_UPDATE.value: TranscriptionsMessageUpdateEvent,
        WebsocketsEventType.TRANSCRIPTIONS_MESSAGE_COMPLETED.value: TranscriptionsMessageCompletedEvent,
    }
)


class WebsocketsAudioTranscriptionsEventHandler(WebsocketsBaseEventHandler):
    # 连接成功
    def on_transcriptions_created(self, cli: "WebsocketsAudioTranscriptionsClient", event: TranscriptionsCreatedEvent):
        pass

    # 配置更新成功
    def on_transcriptions_updated(self, cli: "WebsocketsAudioTranscriptionsClient", event: TranscriptionsUpdatedEvent):
        pass

    # 音频提交完成
    def on_input_audio_buffer_completed(
        self, cli: "WebsocketsAudioTranscriptionsClient", event: InputAudioBufferCompletedEvent
    ):
        pass

    # 音频清除成功
    def on_input_audio_buffer_cleared(
        self, cli: "WebsocketsAudioTranscriptionsClient", event: InputAudioBufferClearedEvent
    ):
        pass

    # 识别出文字
    def on_transcriptions_message_update(
        self, cli: "WebsocketsAudioTranscriptionsClient", event: TranscriptionsMessageUpdateEvent
    ):
        pass

    # 识别完成
    def on_transcriptions_message_completed(
        self, cli: "WebsocketsAudioTranscriptionsClient", event: TranscriptionsMessageCompletedEvent
    ):
        pass


class WebsocketsAudioTranscriptionsClient(WebsocketsBaseClient):
    def __init__(
        self,
        base_url: str,
        requester: Requester,
        on_event: Union[WebsocketsAudioTranscriptionsEventHandler, Dict[WebsocketsEventType, Callable]],
        **kwargs,
    ):
        if isinstance(on_event, WebsocketsAudioTranscriptionsEventHandler):
            on_event = on_event.to_dict()
        super().__init__(
            base_url=base_url,
            requester=requester,
            path="v1/audio/transcriptions",
            event_factory=_audio_transcriptions_event_factory,
            on_event=on_event,  # type: ignore
            wait_events=[WebsocketsEventType.TRANSCRIPTIONS_MESSAGE_COMPLETED],
            **kwargs,
        )

    # 更新语音识别配置
    def transcriptions_update(self, data: TranscriptionsUpdateEvent.Data) -> None:
        self._input_queue.put(TranscriptionsUpdateEvent.model_validate({"data": data}))

    # 流式上传音频片段
    def input_audio_buffer_append(self, data: InputAudioBufferAppendEvent.Data) -> None:
        self._input_queue.put(InputAudioBufferAppendEvent.model_validate({"data": data}))

    # 提交音频
    def input_audio_buffer_complete(self) -> None:
        self._input_queue.put(InputAudioBufferCompleteEvent.model_validate({}))

    # 清除缓冲区音频
    def input_audio_buffer_clear(self) -> None:
        self._input_queue.put(InputAudioBufferClearEvent.model_validate({}))


class WebsocketsAudioTranscriptionsBuildClient(object):
    def __init__(self, base_url: str, requester: Requester):
        self._base_url = remove_url_trailing_slash(base_url)
        self._requester = requester

    def create(
        self,
        *,
        on_event: Union[WebsocketsAudioTranscriptionsEventHandler, Dict[WebsocketsEventType, Callable]],
        **kwargs,
    ) -> WebsocketsAudioTranscriptionsClient:
        return WebsocketsAudioTranscriptionsClient(
            base_url=self._base_url,
            requester=self._requester,
            on_event=on_event,  # type: ignore
            **kwargs,
        )


class AsyncWebsocketsAudioTranscriptionsEventHandler(AsyncWebsocketsBaseEventHandler):
    # 连接成功
    async def on_transcriptions_created(
        self, cli: "AsyncWebsocketsAudioTranscriptionsClient", event: TranscriptionsCreatedEvent
    ):
        pass

    # 配置更新成功
    async def on_transcriptions_updated(
        self, cli: "AsyncWebsocketsAudioTranscriptionsClient", event: TranscriptionsUpdatedEvent
    ):
        pass

    # 音频提交完成
    async def on_input_audio_buffer_completed(
        self, cli: "AsyncWebsocketsAudioTranscriptionsClient", event: InputAudioBufferCompletedEvent
    ):
        pass

    # 音频清除成功
    async def on_input_audio_buffer_cleared(
        self, cli: "AsyncWebsocketsAudioTranscriptionsClient", event: InputAudioBufferClearedEvent
    ):
        pass

    # 识别出文字
    async def on_transcriptions_message_update(
        self, cli: "AsyncWebsocketsAudioTranscriptionsClient", event: TranscriptionsMessageUpdateEvent
    ):
        pass

    # 识别完成
    async def on_transcriptions_message_completed(
        self, cli: "AsyncWebsocketsAudioTranscriptionsClient", event: TranscriptionsMessageCompletedEvent
    ):
        pass


class AsyncWebsocketsAudioTranscriptionsClient(AsyncWebsocketsBaseClient):
    def __init__(
        self,
        base_url: str,
        requester: Requester,
        on_event: Union[AsyncWebsocketsAudioTranscriptionsEventHandler, Dict[WebsocketsEventType, Callable]],
        **kwargs,
    ):
        if isinstance(on_event, AsyncWebsocketsAudioTranscriptionsEventHandler):
            on_event = on_event.to_dict()
        super().__init__(
            base_url=base_url,
            requester=requester,
            path="v1/audio/transcriptions",
            event_factory=_audio_transcriptions_event_factory,
            on_event=on_event,  # type: ignore
            wait_events=[WebsocketsEventType.TRANSCRIPTIONS_MESSAGE_COMPLETED],
            **kwargs,
        )

    # 更新语音识别配置
    async def transcriptions_update(self, data: TranscriptionsUpdateEvent.Data) -> None:
        await self._input_queue.put(TranscriptionsUpdateEvent.model_validate({"data": data}))

    # 流式上传音频片段
    async def input_audio_buffer_append(self, data: InputAudioBufferAppendEvent.Data) -> None:
        await self._input_queue.put(InputAudioBufferAppendEvent.model_validate({"data": data}))

    # 提交音频
    async def input_audio_buffer_complete(self) -> None:
        await self._input_queue.put(InputAudioBufferCompleteEvent.model_validate({}))

    # 清除缓冲区音频
    async def input_audio_buffer_clear(self) -> None:
        await self._input_queue.put(InputAudioBufferClearEvent.model_validate({}))


class AsyncWebsocketsAudioTranscriptionsBuildClient(object):
    def __init__(self, base_url: str, requester: Requester):
        self._base_url = remove_url_trailing_slash(base_url)
        self._requester = requester

    def create(
        self,
        *,
        on_event: Union[AsyncWebsocketsAudioTranscriptionsEventHandler, Dict[WebsocketsEventType, Callable]],
        **kwargs,
    ) -> AsyncWebsocketsAudioTranscriptionsClient:
        return AsyncWebsocketsAudioTranscriptionsClient(
            base_url=self._base_url,
            requester=self._requester,
            on_event=on_event,  # type: ignore
            **kwargs,
        )



from enum import Enum
from typing import List

from cozepy.model import CozeModel
from cozepy.request import Requester
from cozepy.util import remove_url_trailing_slash


class LiveType(str, Enum):
    ORIGIN = "origin"
    TRANSLATION = "translation"


class StreamInfo(CozeModel):
    stream_id: str
    name: str
    live_type: LiveType


class LiveInfo(CozeModel):
    app_id: str
    stream_infos: List[StreamInfo]


class LiveClient(object):
    """
    Live service client.
    """

    def __init__(self, base_url: str, requester: Requester):
        self._base_url = remove_url_trailing_slash(base_url)
        self._requester = requester

    def retrieve(
        self,
        *,
        live_id: str,
    ) -> LiveInfo:
        """
        retrieve live

        :param live_id: The id of the live.
        :return: live info
        """
        url = f"{self._base_url}/v1/audio/live/{live_id}"
        return self._requester.request("get", url, stream=False, cast=LiveInfo)


class AsyncLiveClient(object):
    """
    Room service async client.
    """

    def __init__(self, base_url: str, requester: Requester):
        self._base_url = remove_url_trailing_slash(base_url)
        self._requester = requester

    async def retrieve(
        self,
        *,
        live_id: str,
    ) -> LiveInfo:
        """
        retrieve live

        :param live_id: The id of the live.
        :return: live info
        """
        url = f"{self._base_url}/v1/audio/live/{live_id}"
        return await self._requester.arequest("get", url, stream=False, cast=LiveInfo)



from typing import TYPE_CHECKING, Optional

from cozepy.request import Requester
from cozepy.util import remove_url_trailing_slash

if TYPE_CHECKING:
    from .members import AsyncEnterprisesMembersClient, EnterprisesMembersClient


class EnterprisesClient(object):
    def __init__(self, base_url: str, requester: Requester):
        self._base_url = remove_url_trailing_slash(base_url)
        self._requester = requester

        self._members: Optional[EnterprisesMembersClient] = None

    @property
    def members(self) -> "EnterprisesMembersClient":
        if not self._members:
            from .members import EnterprisesMembersClient

            self._members = EnterprisesMembersClient(self._base_url, self._requester)
        return self._members


class AsyncEnterprisesClient(object):
    def __init__(self, base_url: str, requester: Requester):
        self._base_url = remove_url_trailing_slash(base_url)
        self._requester = requester

        self._members: Optional[AsyncEnterprisesMembersClient] = None

    @property
    def members(self) -> "AsyncEnterprisesMembersClient":
        if not self._members:
            from .members import AsyncEnterprisesMembersClient

            self._members = AsyncEnterprisesMembersClient(self._base_url, self._requester)
        return self._members



from enum import IntEnum
from typing import Dict, List, Optional

from pydantic import Field, field_validator

from cozepy.model import AsyncNumberPaged, CozeModel, DynamicStrEnum, NumberPaged, NumberPagedResponse
from cozepy.request import HTTPRequest, Requester
from cozepy.util import dump_exclude_none, remove_none_values, remove_url_trailing_slash


class PublishStatus(DynamicStrEnum):
    ALL = "all"  # 所有智能体，且数据为最新草稿版本
    PUBLISHED_ONLINE = "published_online"  # 已发布智能体的最新线上版本
    PUBLISHED_DRAFT = "published_draft"  # 已发布的最新草稿版本
    UNPUBLISHED_DRAFT = "unpublished_draft"  # 未发布的最新草稿版本


class BotPromptInfo(CozeModel):
    # Configured prompt of the bot.
    prompt: str


class BotOnboardingInfo(CozeModel):
    # Configured prologue of the bot.
    prologue: str = ""
    # The list of recommended questions configured for the bot.
    # This field will not be returned when user suggested questions are not enabled.
    suggested_questions: List[str] = []


class BotKnowledge(CozeModel):
    # Configured dataset ids of the bot.
    dataset_ids: List[str] = []
    # Whether to call knowledge base automatically.
    auto_call: bool = True
    # Configured search strategy of the bot, values: 0: semantic search, 1: hybrid search, 20: full-text search.
    search_strategy: int = 0


class BotModelInfo(CozeModel):
    class ResponseFormat(DynamicStrEnum):
        JSON = "json"
        TEXT = "text"
        MARKDOWN = "markdown"

    # The ID of the model.
    model_id: str
    # The temperature of the model.
    temperature: Optional[float] = None
    # The context_round of the model.
    context_round: Optional[int] = None
    # The max_tokens of the model.
    max_tokens: Optional[int] = None
    # The response format of the model.
    response_format: Optional[ResponseFormat] = None
    # The top_k of the model.
    top_k: Optional[int] = None
    # The top_p of the model.
    top_p: Optional[float] = None
    # The presence_penalty of the model.
    presence_penalty: Optional[float] = None
    # The frequency_penalty of the model.
    frequency_penalty: Optional[float] = None
    # The parameters of the model.
    # claude: {"thinking_type": "disabled/enable", "thinking_budget_tokens": "2000"}
    # gemini: {"thinking_type": "disabled/enable"}
    # doubao: {"thinking_type": "auto/disabled/enable"}
    parameters: Optional[Dict[str, str]] = None


class BotMode(IntEnum):
    SINGLE_AGENT = 0
    MULTI_AGENT = 1
    SINGLE_AGENT_WORKFLOW = 2


class BotPluginAPIInfo(CozeModel):
    # The ID of the tool.
    api_id: str
    # The name of the tool.
    name: str
    # The description of the tool.
    description: str


class BotPluginInfo(CozeModel):
    # The unique identifier for the plugin.
    plugin_id: str
    # The name of the plugin.
    name: str
    # The description of the plugin.
    description: str
    # The avatar URL for the plugin.
    icon_url: str
    # The list of tool information for the plugin. For more information, see PluginAPI object.
    api_info_list: List[BotPluginAPIInfo]


class SuggestReplyMode(DynamicStrEnum):
    # The bot does not suggest replies.
    DISABLE = "disable"
    # The bot suggests replies.
    ENABLE = "enable"
    # The bot suggests replies based on the customized prompt.
    CUSTOMIZED = "customized"


class BotSuggestReplyInfo(CozeModel):
    reply_mode: SuggestReplyMode
    customized_prompt: str = ""


class GradientPosition(CozeModel):
    left: Optional[float] = None
    right: Optional[float] = None


class CanvasPosition(CozeModel):
    width: Optional[float] = None
    height: Optional[float] = None
    left: Optional[float] = None
    top: Optional[float] = None


class BackgroundImageInfo(CozeModel):
    image_url: str = ""
    theme_color: Optional[str] = None
    gradient_position: Optional[GradientPosition] = None
    canvas_position: Optional[CanvasPosition] = None


class BotBackgroundImageInfo(CozeModel):
    web_background_image: Optional[BackgroundImageInfo] = None
    mobile_background_image: Optional[BackgroundImageInfo] = None


class VariableType(DynamicStrEnum):
    # The variable is a key-value pair.
    KVVariable = "KVVariable"
    # The variable is a list.
    ListVariable = "ListVariable"


class VariableChannel(DynamicStrEnum):
    # The variable is a custom variable.
    VariableChannelCustom = "custom"
    # The variable is a system variable.
    VariableChannelSystem = "system"
    # The variable is a location variable.
    VariableChannelLocation = "location"
    # The variable is a Feishu variable.
    VariableChannelFeishu = "feishu"
    # The variable is an app variable.
    VariableChannelAPP = "app"


class BotVariable(CozeModel):
    # The name of the variable.
    keyword: str
    # The default value of the variable.
    default_value: str
    # The type of the variable.
    variable_type: VariableType
    # The source of the variable.
    channel: VariableChannel
    # The description of the variable.
    description: str
    # Whether the variable is enabled.
    enable: bool
    # Whether the variable is supported in the prompt.
    prompt_enable: bool


class BotVoiceInfo(CozeModel):
    voice_id: str
    language_code: str


class UserInputType(DynamicStrEnum):
    """用户输入类型"""

    # 文本输入
    TEXT = "text"
    # 语音通话
    CALL = "call"
    # 语音输入
    VOICE = "voice"


class BotWorkflowInfo(CozeModel):
    id: str
    name: str
    description: str
    icon_url: str


class Bot(CozeModel):
    # The ID for the bot.
    bot_id: str
    # The name of the bot.
    name: Optional[str] = None
    # The description of the bot.
    description: Optional[str] = None
    # The URL address for the bot's avatar.
    icon_url: Optional[str] = None
    # The creation time, in the format of a 10-digit Unix timestamp in seconds (s).
    create_time: Optional[int] = None
    # The update time, in the format of a 10-digit Unix timestamp in seconds (s).
    update_time: Optional[int] = None
    # The latest version of the bot.
    version: Optional[str] = None
    # The prompt configuration for the bot. For more information, see Prompt object.
    prompt_info: Optional[BotPromptInfo] = None
    # The onboarding message configuration for the bot. For more information, see Onboarding object.
    onboarding_info: Optional[BotOnboardingInfo] = None
    # The knowledge configuration for the bot. For more information, see Knowledge object.
    knowledge: Optional[BotKnowledge] = None
    # The mode of the Bot, values: 0: Single Agent mode, 1: Multi Agent mode, 3: Single Agent Workflow mode.
    bot_mode: Optional[BotMode] = None
    # The plugins configured for the bot. For more information, see  Plugin object.
    plugin_info_list: Optional[List[BotPluginInfo]] = None
    # The model configured for the bot. For more information, see Model object.
    model_info: Optional[BotModelInfo] = None
    # The suggest reply info for the bot.
    suggest_reply_info: Optional[BotSuggestReplyInfo] = None
    # The background image info for the bot.
    background_image_info: Optional[BotBackgroundImageInfo] = None
    # The list of variables configured for the bot. For more information, see Variable object.
    variables: Optional[List[BotVariable]] = None
    # The user ID of the bot's owner.
    owner_user_id: Optional[str] = None
    # The voice info for the bot.
    voice_info_list: Optional[List[BotVoiceInfo]] = None
    # The default user input type for the bot.
    default_user_input_type: Optional[UserInputType] = None
    # The workflow info for the bot.
    workflow_info_list: Optional[List[BotWorkflowInfo]] = None


class SimpleBot(CozeModel):
    id: str
    name: str
    description: str
    icon_url: str
    is_published: bool
    updated_at: int
    owner_user_id: str

    published_at: Optional[int] = None

    # compatibility fields
    bot_id: str = Field(alias="id")
    bot_name: str = Field(alias="name")
    publish_time: str = Field(alias="updated_at")

    @field_validator("publish_time", mode="before")
    @classmethod
    def convert_to_string(cls, v):
        if isinstance(v, int):
            return str(v)
        return v


class UpdateBotResp(CozeModel):
    pass


class _PrivateListBotsDataV1(CozeModel, NumberPagedResponse[SimpleBot]):
    class SimpleBotV1(CozeModel):
        bot_id: str
        bot_name: str
        description: str
        icon_url: str
        publish_time: str

        def to_simple_bot(self) -> SimpleBot:
            return SimpleBot(  # type: ignore[call-arg]
                id=self.bot_id,
                name=self.bot_name,
                description=self.description,
                icon_url=self.icon_url,
                is_published=True,
                updated_at=int(self.publish_time),
                owner_user_id="",
                published_at=int(self.publish_time),
            )

    space_bots: List[SimpleBotV1]
    total: int

    def get_total(self) -> Optional[int]:
        return self.total

    def get_has_more(self) -> Optional[bool]:
        return None

    def get_items(self) -> List[SimpleBot]:
        return [bot.to_simple_bot() for bot in self.space_bots]


class _PrivateListBotsDataV2(CozeModel, NumberPagedResponse[SimpleBot]):
    items: List[SimpleBot]
    total: int

    def get_total(self) -> Optional[int]:
        return self.total

    def get_has_more(self) -> Optional[bool]:
        return None

    def get_items(self) -> List[SimpleBot]:
        return self.items


class BotsClient(object):
    """
    Bot class.
    """

    def __init__(self, base_url: str, requester: Requester):
        self._base_url = remove_url_trailing_slash(base_url)
        self._requester = requester

    def create(
        self,
        *,
        space_id: str,
        name: str,
        description: Optional[str] = None,
        icon_file_id: Optional[str] = None,
        prompt_info: Optional[BotPromptInfo] = None,
        onboarding_info: Optional[BotOnboardingInfo] = None,
        suggest_reply_info: Optional[BotSuggestReplyInfo] = None,
        model_info_config: Optional[BotModelInfo] = None,
        **kwargs,
    ) -> Bot:
        url = f"{self._base_url}/v1/bot/create"
        body = dump_exclude_none(
            {
                "space_id": space_id,
                "name": name,
                "description": description,
                "icon_file_id": icon_file_id,
                "prompt_info": prompt_info,
                "onboarding_info": onboarding_info,
                "suggest_reply_info": suggest_reply_info,
                "model_info_config": model_info_config,
            }
        )
        headers: Optional[dict] = kwargs.get("headers")

        return self._requester.request("post", url, False, Bot, body=body, headers=headers)

    def update(
        self,
        *,
        bot_id: str,
        name: Optional[str] = None,
        description: Optional[str] = None,
        icon_file_id: Optional[str] = None,
        prompt_info: Optional[BotPromptInfo] = None,
        onboarding_info: Optional[BotOnboardingInfo] = None,
        knowledge: Optional[BotKnowledge] = None,
        suggest_reply_info: Optional[BotSuggestReplyInfo] = None,
        model_info_config: Optional[BotModelInfo] = None,
        **kwargs,
    ) -> UpdateBotResp:
        """
        Update the configuration of a bot.
        This API can be used to update all bots created through the Coze platform or via the API.
        In addition to updating the bot's name and description, avatar, personality and reply logic,
        and opening remarks, this API also supports binding a knowledge base to the bot.

        docs en: https://www.coze.com/docs/developer_guides/update_bot
        docs zh: https://www.coze.cn/docs/developer_guides/update_bot

        :param bot_id: The ID of the bot that the API interacts with.
        :param name: The name of the bot. It should be 1 to 20 characters long.
        :param description: The description of the Bot. It can be 0 to 500 characters long. The default is empty.
        :param icon_file_id: The file ID for the Bot's avatar. If no file ID is specified, the Coze platform will
        assign a default avatar for the bot. To use a custom avatar, first upload the local file through the Upload
        file interface and obtain the file ID from the interface response.
        :param prompt_info: The personality and reply logic of the bot.
        :param onboarding_info: The settings related to the bot's opening remarks.
        :param knowledge: The knowledge base that the bot uses to answer user queries.
        :param suggest_reply_info: The suggest reply info for the bot.
        :param model_info_config: The model configuration for the bot.
        :return: None
        """
        url = f"{self._base_url}/v1/bot/update"
        body = dump_exclude_none(
            {
                "bot_id": bot_id,
                "name": name,
                "description": description,
                "icon_file_id": icon_file_id,
                "prompt_info": prompt_info,
                "onboarding_info": onboarding_info,
                "knowledge": knowledge,
                "suggest_reply_info": suggest_reply_info,
                "model_info_config": model_info_config,
            }
        )
        headers: Optional[dict] = kwargs.get("headers")

        return self._requester.request(
            "post",
            url,
            False,
            cast=UpdateBotResp,
            body=body,
            headers=headers,
        )

    def publish(self, *, bot_id: str, connector_ids: Optional[List[str]] = None, **kwargs) -> Bot:
        url = f"{self._base_url}/v1/bot/publish"
        headers: Optional[dict] = kwargs.get("headers")
        if not connector_ids:
            connector_ids = ["1024"]
        body = {
            "bot_id": bot_id,
            "connector_ids": connector_ids,
        }

        return self._requester.request("post", url, False, Bot, headers=headers, body=body)

    def retrieve(self, *, bot_id: str, is_published: Optional[bool] = None, use_api_version: int = 1, **kwargs) -> Bot:
        """查看智能体配置

        查看指定智能体的配置信息，你可以查看该智能体已发布版本的配置，或当前草稿版本的配置。

        docs: https://www.coze.cn/open/docs/developer_guides/get_metadata_draft_published

        :param bot_id: 要查看的智能体 ID。
        :param is_published: 根据智能体的发布状态筛选对应版本。默认值为 true。
        """
        if use_api_version == 2:
            return self._retrieve_v2(bot_id=bot_id, is_published=is_published, **kwargs)
        else:
            return self._retrieve_v1(bot_id=bot_id, **kwargs)

    def _retrieve_v1(self, *, bot_id: str, **kwargs) -> Bot:
        """
        Get the configuration information of the bot, which must have been published
        to the Bot as API channel.
        获取指定 Bot 的配置信息，此 Bot 必须已发布到 Bot as API 渠道中。

        docs en: https://www.coze.com/docs/developer_guides/get_metadata
        docs zh: https://www.coze.cn/docs/developer_guides/get_metadata

        :param bot_id: The ID of the bot that the API interacts with.
        要查看的 Bot ID。
        :return: bot object
        Bot 的配置信息
        """
        url = f"{self._base_url}/v1/bot/get_online_info"
        params = {"bot_id": bot_id}
        headers: Optional[dict] = kwargs.get("headers")

        return self._requester.request("get", url, False, Bot, params=params, headers=headers)

    def _retrieve_v2(self, *, bot_id: str, is_published: Optional[bool] = None, **kwargs) -> Bot:
        """查看智能体配置

        查看指定智能体的配置信息，你可以查看该智能体已发布版本的配置，或当前草稿版本的配置。

        docs: https://www.coze.cn/open/docs/developer_guides/get_metadata_draft_published

        :param bot_id: 要查看的智能体 ID。
        :param is_published: 根据智能体的发布状态筛选对应版本。默认值为 true。
        """
        url = f"{self._base_url}/v1/bots/{bot_id}"
        params = remove_none_values({"is_published": is_published})
        headers: Optional[dict] = kwargs.get("headers")

        return self._requester.request("get", url, False, Bot, params=params, headers=headers)

    def list(
        self,
        *,
        space_id: str,
        publish_status: Optional[PublishStatus] = None,
        connector_id: Optional[str] = None,
        page_num: int = 1,
        page_size: int = 20,
        use_api_version: int = 1,
        **kwargs,
    ) -> NumberPaged[SimpleBot]:
        """
        查看指定空间的智能体列表

        docs en: https://www.coze.com/docs/developer_guides/published_bots_list
        docs zh: https://www.coze.cn/open/docs/developer_guides/bots_list_draft_published

        :param space_id: The ID of the space.
        Bot 所在的空间的 Space ID。Space ID 是空间的唯一标识。
        :param page_num: Pagination size. The default is 20, meaning that 20 data entries are returned per page.
        分页大小。默认为 20，即每页返回 20 条数据。
        :param page_size: Page number for paginated queries. The default is 1,
        meaning that the data return starts from the first page.
        分页查询时的页码。默认为 1，即从第一页数据开始返回。
        :return: Specify the list of Bots published to the Bot as an API channel in space.
        指定空间发布到 Bot as API 渠道的 Bot 列表。
        """
        if use_api_version == 2:
            return self._list_v2(
                workspace_id=space_id,
                publish_status=publish_status,
                connector_id=connector_id,
                page_num=page_num,
                page_size=page_size,
            )
        else:
            return self._list_v1(
                space_id=space_id,
                page_num=page_num,
                page_size=page_size,
            )

    def _list_v1(self, *, space_id: str, page_num: int = 1, page_size: int = 20) -> NumberPaged[SimpleBot]:
        """
        Get the bots published as API service.
        查看指定空间发布到 Bot as API 渠道的 Bot 列表。

        docs en: https://www.coze.com/docs/developer_guides/published_bots_list
        docs zh: https://www.coze.cn/docs/developer_guides/published_bots_list

        :param space_id: The ID of the space.
        Bot 所在的空间的 Space ID。Space ID 是空间的唯一标识。
        :param page_num: Pagination size. The default is 20, meaning that 20 data entries are returned per page.
        分页大小。默认为 20，即每页返回 20 条数据。
        :param page_size: Page number for paginated queries. The default is 1,
        meaning that the data return starts from the first page.
        分页查询时的页码。默认为 1，即从第一页数据开始返回。
        :return: Specify the list of Bots published to the Bot as an API channel in space.
        指定空间发布到 Bot as API 渠道的 Bot 列表。
        """
        url = f"{self._base_url}/v1/space/published_bots_list"

        def request_maker(i_page_num: int, i_page_size: int) -> HTTPRequest:
            return self._requester.make_request(
                "GET",
                url,
                params={
                    "space_id": space_id,
                    "page_size": i_page_size,
                    "page_index": i_page_num,
                },
                cast=_PrivateListBotsDataV1,
                stream=False,
            )

        return NumberPaged(
            page_num=page_num,
            page_size=page_size,
            requestor=self._requester,
            request_maker=request_maker,
        )

    def _list_v2(
        self,
        *,
        workspace_id: str,
        publish_status: Optional[PublishStatus] = None,
        connector_id: Optional[str] = None,
        page_num: int = 1,
        page_size: int = 20,
        **kwargs,
    ) -> NumberPaged[SimpleBot]:
        """
        查看指定空间的智能体列表

        docs en: https://www.coze.com/docs/developer_guides/published_bots_list
        docs zh: https://www.coze.cn/open/docs/developer_guides/bots_list_draft_published

        :param workspace_id: The ID of the space.
        Bot 所在的空间的 Space ID。Space ID 是空间的唯一标识。
        :param page_num: Pagination size. The default is 20, meaning that 20 data entries are returned per page.
        分页大小。默认为 20，即每页返回 20 条数据。
        :param page_size: Page number for paginated queries. The default is 1,
        meaning that the data return starts from the first page.
        分页查询时的页码。默认为 1，即从第一页数据开始返回。
        :return: Specify the list of Bots published to the Bot as an API channel in space.
        指定空间发布到 Bot as API 渠道的 Bot 列表。
        """
        url = f"{self._base_url}/v1/bots"
        headers: Optional[dict] = kwargs.get("headers")

        def request_maker(i_page_num: int, i_page_size: int) -> HTTPRequest:
            return self._requester.make_request(
                "GET",
                url,
                params=remove_none_values(
                    {
                        "workspace_id": workspace_id,
                        "page_size": i_page_size,
                        "page_index": i_page_num,
                        "publish_status": publish_status.value if publish_status else None,
                        "connector_id": connector_id,
                    }
                ),
                headers=headers,
                cast=_PrivateListBotsDataV2,
                stream=False,
            )

        return NumberPaged(
            page_num=page_num,
            page_size=page_size,
            requestor=self._requester,
            request_maker=request_maker,
        )


class AsyncBotsClient(object):
    """
    Bot class.
    """

    def __init__(self, base_url: str, requester: Requester):
        self._base_url = remove_url_trailing_slash(base_url)
        self._requester = requester

    async def create(
        self,
        *,
        space_id: str,
        name: str,
        description: Optional[str] = None,
        icon_file_id: Optional[str] = None,
        prompt_info: Optional[BotPromptInfo] = None,
        onboarding_info: Optional[BotOnboardingInfo] = None,
        suggest_reply_info: Optional[BotSuggestReplyInfo] = None,
        model_info_config: Optional[BotModelInfo] = None,
        **kwargs,
    ) -> Bot:
        url = f"{self._base_url}/v1/bot/create"
        body = dump_exclude_none(
            {
                "space_id": space_id,
                "name": name,
                "description": description,
                "icon_file_id": icon_file_id,
                "prompt_info": prompt_info,
                "onboarding_info": onboarding_info,
                "suggest_reply_info": suggest_reply_info,
                "model_info_config": model_info_config,
            }
        )
        headers: Optional[dict] = kwargs.get("headers")

        return await self._requester.arequest("post", url, False, Bot, body=body, headers=headers)

    async def update(
        self,
        *,
        bot_id: str,
        name: Optional[str] = None,
        description: Optional[str] = None,
        icon_file_id: Optional[str] = None,
        prompt_info: Optional[BotPromptInfo] = None,
        onboarding_info: Optional[BotOnboardingInfo] = None,
        knowledge: Optional[BotKnowledge] = None,
        suggest_reply_info: Optional[BotSuggestReplyInfo] = None,
        model_info_config: Optional[BotModelInfo] = None,
        **kwargs,
    ) -> UpdateBotResp:
        """
        Update the configuration of a bot.
        This API can be used to update all bots created through the Coze platform or via the API.
        In addition to updating the bot's name and description, avatar, personality and reply logic,
        and opening remarks, this API also supports binding a knowledge base to the bot.

        docs en: https://www.coze.com/docs/developer_guides/update_bot
        docs zh: https://www.coze.cn/docs/developer_guides/update_bot

        :param bot_id: The ID of the bot that the API interacts with.
        :param name: The name of the bot. It should be 1 to 20 characters long.
        :param description: The description of the Bot. It can be 0 to 500 characters long. The default is empty.
        :param icon_file_id: The file ID for the Bot's avatar. If no file ID is specified, the Coze platform will
        assign a default avatar for the bot. To use a custom avatar, first upload the local file through the Upload
        file interface and obtain the file ID from the interface response.
        :param prompt_info: The personality and reply logic of the bot.
        :param onboarding_info: The settings related to the bot's opening remarks.
        :param knowledge: The knowledge base that the bot uses to answer user queries.
        :param suggest_reply_info: The suggest reply info for the bot.
        :param model_info_config: The model configuration for the bot.
        :return: None
        """
        url = f"{self._base_url}/v1/bot/update"
        body = dump_exclude_none(
            {
                "bot_id": bot_id,
                "name": name,
                "description": description,
                "icon_file_id": icon_file_id,
                "prompt_info": prompt_info,
                "onboarding_info": onboarding_info,
                "knowledge": knowledge,
                "suggest_reply_info": suggest_reply_info,
                "model_info_config": model_info_config,
            }
        )
        headers: Optional[dict] = kwargs.get("headers")

        return await self._requester.arequest("post", url, False, cast=UpdateBotResp, body=body, headers=headers)

    async def publish(
        self,
        *,
        bot_id: str,
        connector_ids: Optional[List[str]] = None,
        **kwargs,
    ) -> Bot:
        url = f"{self._base_url}/v1/bot/publish"
        headers: Optional[dict] = kwargs.get("headers")
        if not connector_ids:
            connector_ids = ["1024"]
        body = {
            "bot_id": bot_id,
            "connector_ids": connector_ids,
        }

        return await self._requester.arequest("post", url, False, Bot, headers=headers, body=body)

    async def retrieve(
        self, *, bot_id: str, is_published: Optional[bool] = None, use_api_version: int = 1, **kwargs
    ) -> Bot:
        """查看智能体配置

        查看指定智能体的配置信息，你可以查看该智能体已发布版本的配置，或当前草稿版本的配置。

        docs: https://www.coze.cn/open/docs/developer_guides/get_metadata_draft_published

        :param bot_id: 要查看的智能体 ID。
        :param is_published: 根据智能体的发布状态筛选对应版本。默认值为 true。
        """
        if use_api_version == 2:
            return await self._retrieve_v2(bot_id=bot_id, is_published=is_published, **kwargs)
        else:
            return await self._retrieve_v1(bot_id=bot_id, **kwargs)

    async def _retrieve_v1(self, *, bot_id: str, **kwargs) -> Bot:
        """
        Get the configuration information of the bot, which must have been published
        to the Bot as API channel.
        获取指定 Bot 的配置信息，此 Bot 必须已发布到 Bot as API 渠道中。

        docs en: https://www.coze.com/docs/developer_guides/get_metadata
        docs zh: https://www.coze.cn/docs/developer_guides/get_metadata

        :param bot_id: The ID of the bot that the API interacts with.
        要查看的 Bot ID。
        :return: bot object
        Bot 的配置信息
        """
        url = f"{self._base_url}/v1/bot/get_online_info"
        params = {"bot_id": bot_id}
        headers: Optional[dict] = kwargs.get("headers")

        return await self._requester.arequest("get", url, False, Bot, params=params, headers=headers)

    async def _retrieve_v2(self, *, bot_id: str, is_published: Optional[bool] = None, **kwargs) -> Bot:
        """查看智能体配置

        查看指定智能体的配置信息，你可以查看该智能体已发布版本的配置，或当前草稿版本的配置。

        docs: https://www.coze.cn/open/docs/developer_guides/get_metadata_draft_published

        :param bot_id: 要查看的智能体 ID。
        :param is_published: 根据智能体的发布状态筛选对应版本。默认值为 true。
        """
        url = f"{self._base_url}/v1/bots/{bot_id}"
        params = remove_none_values({"is_published": is_published})
        headers: Optional[dict] = kwargs.get("headers")

        return await self._requester.arequest("get", url, False, Bot, params=params, headers=headers)

    async def list(
        self,
        *,
        space_id: str,
        publish_status: Optional[PublishStatus] = None,
        connector_id: Optional[str] = None,
        page_num: int = 1,
        page_size: int = 20,
        use_api_version: int = 1,
        **kwargs,
    ) -> AsyncNumberPaged[SimpleBot]:
        """
        查看指定空间的智能体列表

        docs en: https://www.coze.com/docs/developer_guides/published_bots_list
        docs zh: https://www.coze.cn/open/docs/developer_guides/bots_list_draft_published

        :param space_id: The ID of the space.
        Bot 所在的空间的 Space ID。Space ID 是空间的唯一标识。
        :param page_num: Pagination size. The default is 20, meaning that 20 data entries are returned per page.
        分页大小。默认为 20，即每页返回 20 条数据。
        :param page_size: Page number for paginated queries. The default is 1,
        meaning that the data return starts from the first page.
        分页查询时的页码。默认为 1，即从第一页数据开始返回。
        :return: Specify the list of Bots published to the Bot as an API channel in space.
        指定空间发布到 Bot as API 渠道的 Bot 列表。
        """
        if use_api_version == 2:
            return await self._list_v2(
                workspace_id=space_id,
                publish_status=publish_status,
                connector_id=connector_id,
                page_num=page_num,
                page_size=page_size,
                **kwargs,
            )
        else:
            return await self._list_v1(space_id=space_id, page_num=page_num, page_size=page_size, **kwargs)

    async def _list_v1(
        self, *, space_id: str, page_num: int = 1, page_size: int = 20, **kwargs
    ) -> AsyncNumberPaged[SimpleBot]:
        """
        Get the bots published as API service.
        查看指定空间发布到 Bot as API 渠道的 Bot 列表。

        docs en: https://www.coze.com/docs/developer_guides/published_bots_list
        docs zh: https://www.coze.cn/docs/developer_guides/published_bots_list

        :param space_id: The ID of the space.
        Bot 所在的空间的 Space ID。Space ID 是空间的唯一标识。
        :param page_num: Pagination size. The default is 20, meaning that 20 data entries are returned per page.
        分页大小。默认为 20，即每页返回 20 条数据。
        :param page_size: Page number for paginated queries. The default is 1,
        meaning that the data return starts from the first page.
        分页查询时的页码。默认为 1，即从第一页数据开始返回。
        :return: Specify the list of Bots published to the Bot as an API channel in space.
        指定空间发布到 Bot as API 渠道的 Bot 列表。
        """
        url = f"{self._base_url}/v1/space/published_bots_list"

        async def request_maker(i_page_num: int, i_page_size: int) -> HTTPRequest:
            return await self._requester.amake_request(
                "GET",
                url,
                params={
                    "space_id": space_id,
                    "page_size": i_page_size,
                    "page_index": i_page_num,
                },
                cast=_PrivateListBotsDataV1,
                stream=False,
            )

        return await AsyncNumberPaged.build(
            page_num=page_num,
            page_size=page_size,
            requestor=self._requester,
            request_maker=request_maker,
        )

    async def _list_v2(
        self,
        *,
        workspace_id: str,
        publish_status: Optional[PublishStatus] = None,
        connector_id: Optional[str] = None,
        page_num: int = 1,
        page_size: int = 20,
        **kwargs,
    ) -> AsyncNumberPaged[SimpleBot]:
        """
        查看指定空间的智能体列表

        docs en: https://www.coze.com/docs/developer_guides/published_bots_list
        docs zh: https://www.coze.cn/open/docs/developer_guides/bots_list_draft_published

        :param workspace_id: The ID of the space.
        Bot 所在的空间的 Space ID。Space ID 是空间的唯一标识。
        :param page_num: Pagination size. The default is 20, meaning that 20 data entries are returned per page.
        分页大小。默认为 20，即每页返回 20 条数据。
        :param page_size: Page number for paginated queries. The default is 1,
        meaning that the data return starts from the first page.
        分页查询时的页码。默认为 1，即从第一页数据开始返回。
        :return: Specify the list of Bots published to the Bot as an API channel in space.
        指定空间发布到 Bot as API 渠道的 Bot 列表。
        """
        url = f"{self._base_url}/v1/bots"
        headers: Optional[dict] = kwargs.get("headers")

        async def request_maker(i_page_num: int, i_page_size: int) -> HTTPRequest:
            return await self._requester.amake_request(
                "GET",
                url,
                params=remove_none_values(
                    {
                        "workspace_id": workspace_id,
                        "page_size": i_page_size,
                        "page_index": i_page_num,
                        "publish_status": publish_status.value if publish_status else None,
                        "connector_id": connector_id,
                    }
                ),
                headers=headers,
                cast=_PrivateListBotsDataV2,
                stream=False,
            )

        return await AsyncNumberPaged.build(
            page_num=page_num,
            page_size=page_size,
            requestor=self._requester,
            request_maker=request_maker,
        )



import base64
from typing import Callable, Dict, Optional, Union

from pydantic import BaseModel, field_serializer, field_validator

from cozepy.request import Requester
from cozepy.util import remove_url_trailing_slash
from cozepy.websockets.ws import (
    AsyncWebsocketsBaseClient,
    AsyncWebsocketsBaseEventHandler,
    OutputAudio,
    WebsocketsBaseClient,
    WebsocketsBaseEventHandler,
    WebsocketsEvent,
    WebsocketsEventFactory,
    WebsocketsEventType,
)


# req
class InputTextBufferAppendEvent(WebsocketsEvent):
    """流式输入文字

    流式向服务端提交文字的片段。
    docs: https://www.coze.cn/open/docs/developer_guides/tts_event#0ba93be3
    """

    class Data(BaseModel):
        # 需要合成语音的文字片段。
        delta: str

    event_type: WebsocketsEventType = WebsocketsEventType.INPUT_TEXT_BUFFER_APPEND
    data: Data


# req
class InputTextBufferCompleteEvent(WebsocketsEvent):
    """提交文字

    提交 append 的文本，发送后将收到 input_text_buffer.completed 的下行事件。
    docs: https://www.coze.cn/open/docs/developer_guides/tts_event#ab24ada9
    """

    event_type: WebsocketsEventType = WebsocketsEventType.INPUT_TEXT_BUFFER_COMPLETE


# req
class SpeechUpdateEvent(WebsocketsEvent):
    """更新语音合成配置

    更新流式语音合成配置。
    docs: https://www.coze.cn/open/docs/developer_guides/tts_event#6166c24c
    """

    class Data(BaseModel):
        # 输出音频格式。
        output_audio: Optional[OutputAudio] = None

    event_type: WebsocketsEventType = WebsocketsEventType.SPEECH_UPDATE
    data: Data


# resp
class SpeechCreatedEvent(WebsocketsEvent):
    """语音合成连接成功

    语音合成连接成功后，返回此事件。
    docs: https://www.coze.cn/open/docs/developer_guides/tts_event#23c0993e
    """

    event_type: WebsocketsEventType = WebsocketsEventType.SPEECH_CREATED


# resp
class SpeechUpdatedEvent(WebsocketsEvent):
    """配置更新完成

    配置更新成功后，会返回最新的配置。
    docs: https://www.coze.cn/open/docs/developer_guides/tts_event#a3a59fb4
    """

    class Data(BaseModel):
        # 输出音频格式。
        output_audio: Optional[OutputAudio] = None

    event_type: WebsocketsEventType = WebsocketsEventType.SPEECH_UPDATED
    data: Data


# resp
class InputTextBufferCompletedEvent(WebsocketsEvent):
    """input_text_buffer 提交完成

    流式提交的文字完成后，返回此事件。
    docs: https://www.coze.cn/open/docs/developer_guides/tts_event#cf5e0495
    """

    event_type: WebsocketsEventType = WebsocketsEventType.INPUT_TEXT_BUFFER_COMPLETED


# resp
class SpeechAudioUpdateEvent(WebsocketsEvent):
    """合成增量语音

    语音合成产生增量语音时，返回此事件。
    docs: https://www.coze.cn/open/docs/developer_guides/tts_event#98163c71
    """

    class Data(BaseModel):
        # 音频片段。(API 返回的是base64编码的音频片段, SDK 已经自动解码为 bytes)
        delta: bytes

        @field_serializer("delta")
        def serialize_delta(self, delta: bytes, _info):
            return base64.b64encode(delta)

        @field_validator("delta", mode="before")
        @classmethod
        def validate_delta(cls, delta: str):
            return base64.b64decode(delta)

    event_type: WebsocketsEventType = WebsocketsEventType.SPEECH_AUDIO_UPDATE
    data: Data


# resp
class SpeechAudioCompletedEvent(WebsocketsEvent):
    """合成完成

    语音合成完成后，返回此事件。
    docs: https://www.coze.cn/open/docs/developer_guides/tts_event#f42e9cb7
    """

    event_type: WebsocketsEventType = WebsocketsEventType.SPEECH_AUDIO_COMPLETED


_audio_speech_event_factory = WebsocketsEventFactory(
    {
        WebsocketsEventType.SPEECH_CREATED.value: SpeechCreatedEvent,
        WebsocketsEventType.SPEECH_UPDATED.value: SpeechUpdatedEvent,
        WebsocketsEventType.INPUT_TEXT_BUFFER_COMPLETED.value: InputTextBufferCompletedEvent,
        WebsocketsEventType.SPEECH_AUDIO_UPDATE.value: SpeechAudioUpdateEvent,
        WebsocketsEventType.SPEECH_AUDIO_COMPLETED.value: SpeechAudioCompletedEvent,
    }
)


class WebsocketsAudioSpeechEventHandler(WebsocketsBaseEventHandler):
    # 语音合成连接成功
    def on_speech_created(self, cli: "WebsocketsAudioSpeechClient", event: SpeechCreatedEvent):
        pass

    # 配置更新完成
    def on_speech_updated(self, cli: "WebsocketsAudioSpeechClient", event: SpeechUpdatedEvent):
        pass

    # input_text_buffer 提交完成
    def on_input_text_buffer_completed(self, cli: "WebsocketsAudioSpeechClient", event: InputTextBufferCompletedEvent):
        pass

    # 合成增量语音
    def on_speech_audio_update(self, cli: "WebsocketsAudioSpeechClient", event: SpeechAudioUpdateEvent):
        pass

    # 合成完成
    def on_speech_audio_completed(self, cli: "WebsocketsAudioSpeechClient", event: SpeechAudioCompletedEvent):
        pass


class WebsocketsAudioSpeechClient(WebsocketsBaseClient):
    def __init__(
        self,
        base_url: str,
        requester: Requester,
        on_event: Union[WebsocketsAudioSpeechEventHandler, Dict[WebsocketsEventType, Callable]],
        **kwargs,
    ):
        if isinstance(on_event, WebsocketsAudioSpeechEventHandler):
            on_event = on_event.to_dict()
        super().__init__(
            base_url=base_url,
            requester=requester,
            path="v1/audio/speech",
            event_factory=_audio_speech_event_factory,
            on_event=on_event,  # type: ignore
            wait_events=[WebsocketsEventType.SPEECH_AUDIO_COMPLETED],
            **kwargs,
        )

    # 更新语音合成配置
    def speech_update(self, event: SpeechUpdateEvent) -> None:
        self._input_queue.put(event)

    # 流式输入文字
    def input_text_buffer_append(self, data: InputTextBufferAppendEvent.Data) -> None:
        self._input_queue.put(InputTextBufferAppendEvent.model_validate({"data": data}))

    # 提交文字
    def input_text_buffer_complete(self) -> None:
        self._input_queue.put(InputTextBufferCompleteEvent.model_validate({}))


class WebsocketsAudioSpeechBuildClient(object):
    def __init__(self, base_url: str, requester: Requester):
        self._base_url = remove_url_trailing_slash(base_url)
        self._requester = requester

    def create(
        self, *, on_event: Union[WebsocketsAudioSpeechEventHandler, Dict[WebsocketsEventType, Callable]], **kwargs
    ) -> WebsocketsAudioSpeechClient:
        return WebsocketsAudioSpeechClient(
            base_url=self._base_url,
            requester=self._requester,
            on_event=on_event,  # type: ignore
            **kwargs,
        )


class AsyncWebsocketsAudioSpeechEventHandler(AsyncWebsocketsBaseEventHandler):
    # 语音合成连接成功
    async def on_speech_created(self, cli: "AsyncWebsocketsAudioSpeechClient", event: SpeechCreatedEvent):
        pass

    # 配置更新完成
    async def on_speech_updated(self, cli: "AsyncWebsocketsAudioSpeechClient", event: SpeechUpdatedEvent):
        pass

    # input_text_buffer 提交完成
    async def on_input_text_buffer_completed(
        self, cli: "AsyncWebsocketsAudioSpeechClient", event: InputTextBufferCompletedEvent
    ):
        pass

    # 合成增量语音
    async def on_speech_audio_update(self, cli: "AsyncWebsocketsAudioSpeechClient", event: SpeechAudioUpdateEvent):
        pass

    # 合成完成
    async def on_speech_audio_completed(
        self, cli: "AsyncWebsocketsAudioSpeechClient", event: SpeechAudioCompletedEvent
    ):
        pass


class AsyncWebsocketsAudioSpeechClient(AsyncWebsocketsBaseClient):
    def __init__(
        self,
        base_url: str,
        requester: Requester,
        on_event: Union[AsyncWebsocketsAudioSpeechEventHandler, Dict[WebsocketsEventType, Callable]],
        **kwargs,
    ):
        if isinstance(on_event, AsyncWebsocketsAudioSpeechEventHandler):
            on_event = on_event.to_dict()
        super().__init__(
            base_url=base_url,
            requester=requester,
            path="v1/audio/speech",
            event_factory=_audio_speech_event_factory,
            on_event=on_event,  # type: ignore
            wait_events=[WebsocketsEventType.SPEECH_AUDIO_COMPLETED],
            **kwargs,
        )

    # 更新语音合成配置
    async def speech_update(self, event: SpeechUpdateEvent) -> None:
        await self._input_queue.put(event)

    # 流式输入文字
    async def input_text_buffer_append(self, data: InputTextBufferAppendEvent.Data) -> None:
        await self._input_queue.put(InputTextBufferAppendEvent.model_validate({"data": data}))

    # 提交文字
    async def input_text_buffer_complete(self) -> None:
        await self._input_queue.put(InputTextBufferCompleteEvent.model_validate({}))


class AsyncWebsocketsAudioSpeechBuildClient(object):
    def __init__(self, base_url: str, requester: Requester):
        self._base_url = remove_url_trailing_slash(base_url)
        self._requester = requester

    def create(
        self,
        *,
        on_event: Union[AsyncWebsocketsAudioSpeechEventHandler, Dict[WebsocketsEventType, Callable]],
        **kwargs,
    ) -> AsyncWebsocketsAudioSpeechClient:
        return AsyncWebsocketsAudioSpeechClient(
            base_url=self._base_url,
            requester=self._requester,
            on_event=on_event,  # type: ignore
            **kwargs,
        )



from typing import Any, Callable, Dict, List, Optional, Union

from pydantic import BaseModel

from cozepy import Chat, Message, ToolOutput
from cozepy.model import DynamicStrEnum
from cozepy.request import Requester
from cozepy.util import remove_none_values, remove_url_trailing_slash
from cozepy.websockets.audio.transcriptions import (
    InputAudioBufferAppendEvent,
    InputAudioBufferClearedEvent,
    InputAudioBufferClearEvent,
    InputAudioBufferCompletedEvent,
    InputAudioBufferCompleteEvent,
)
from cozepy.websockets.ws import (
    AsyncWebsocketsBaseClient,
    AsyncWebsocketsBaseEventHandler,
    InputAudio,
    OutputAudio,
    WebsocketsBaseClient,
    WebsocketsBaseEventHandler,
    WebsocketsEvent,
    WebsocketsEventFactory,
    WebsocketsEventType,
)


# req
class ChatUpdateEvent(WebsocketsEvent):
    """更新对话配置

    此事件可以更新当前对话连接的配置项，若更新成功，会收到 chat.updated 的下行事件，否则，会收到 error 下行事件。
    docs: https://www.coze.cn/open/docs/developer_guides/streaming_chat_event#91642fa8
    """

    class ChatConfig(BaseModel):
        # 标识对话发生在哪一次会话中。会话是智能体和用户之间的一段问答交互。一个会话包含一条或多条消息。对话是会话中对智能体的一次调用，智能体会将对话中产生的消息添加到会话中。可以使用已创建的会话，会话中已存在的消息将作为上下文传递给模型。创建会话的方式可参考创建会话。对于一问一答等不需要区分 conversation 的场合可不传该参数，系统会自动生成一个会话。不传的话会默认创建一个新的 conversation。
        conversation_id: Optional[str] = None
        # 标识当前与智能体的用户，由使用方自行定义、生成与维护。user_id 用于标识对话中的不同用户，不同的 user_id，其对话的上下文消息、数据库等对话记忆数据互相隔离。如果不需要用户数据隔离，可将此参数固定为一个任意字符串，例如 123，abc 等。
        user_id: Optional[str] = None
        # 附加信息，通常用于封装一些业务相关的字段。查看对话消息详情时，系统会透传此附加信息。自定义键值对，应指定为 Map 对象格式。长度为 16 对键值对，其中键（key）的长度范围为 1～64 个字符，值（value）的长度范围为 1～512 个字符。
        meta_data: Optional[Dict[str, str]] = None
        # 智能体中定义的变量。在智能体 prompt 中设置变量 {{key}} 后，可以通过该参数传入变量值，同时支持 Jinja2 语法。详细说明可参考变量示例。变量名只支持英文字母和下划线。
        custom_variables: Optional[Dict[str, str]] = None
        # 附加参数，通常用于特殊场景下指定一些必要参数供模型判断，例如指定经纬度，并询问智能体此位置的天气。自定义键值对格式，其中键（key）仅支持设置为：latitude（纬度，此时值（Value）为纬度值，例如 39.9800718）。longitude（经度，此时值（Value）为经度值，例如 116.309314）。
        extra_params: Optional[Dict[str, str]] = None
        # 是否保存本次对话记录。true：（默认）会话中保存本次对话记录，包括本次对话的模型回复结果、模型执行中间结果。false：会话中不保存本次对话记录，后续也无法通过任何方式查看本次对话信息、消息详情。在同一个会话中再次发起对话时，本次会话也不会作为上下文传递给模型。
        auto_save_history: Optional[bool] = None
        # 设置对话流的自定义输入参数的值，具体用法和示例代码可参考[为自定义参数赋值](https://www.coze.cn/open/docs/tutorial/variable)。 对话流的输入参数 USER_INPUT 应在 additional_messages 中传入，在 parameters 中的 USER_INPUT 不生效。 如果 parameters 中未指定 CONVERSATION_NAME 或其他输入参数，则使用参数默认值运行对话流；如果指定了这些参数，则使用指定值。
        parameters: Optional[Dict[str, Any]] = None

    class TurnDetectionType(DynamicStrEnum):
        # server_vad ：自由对话模式，语音数据会传输到服务器端进行实时分析，服务器端的语音活动检测算法会判断用户是否在说话。
        SERVER_VAD = "server_vad"
        # client_interrupt：（默认）按键说话模式，客户端实时分析语音数据，并检测用户是否已停止说话。
        CLIENT_INTERRUPT = "client_interrupt"

    class InterruptConfigMode(DynamicStrEnum):
        # keyword_contains模式下，说话内容包含关键词才会打断模型回复。例如关键词"扣子"，用户正在说“你好呀扣子......” / “扣子你好呀”，模型回复都会被打断。
        KEYWORD_CONTAINS = "keyword_contains"
        # keyword_prefix模式下，说话内容前缀匹配关键词才会打断模型回复。例如关键词"扣子"，用户正在说“扣子你好呀......”，模型回复就会被打断，而用户说“你好呀扣子......”，模型回复不会被打断。
        KEYWORD_PREFIX = "keyword_prefix"

    class ASRConfigUserLanguage(DynamicStrEnum):
        COMMON = "common"  # 大模型语音识别，可自动识别中英粤。
        ZH = "zh"  # 小模型语音识别，中文。
        CANT = "cant"  # 小模型语音识别，粤语。
        SC = "sc"  # 小模型语音识别，川渝。
        EN = "en"  # 小模型语音识别，英语。
        JA = "ja"  # 小模型语音识别，日语。
        KO = "ko"  # 小模型语音识别，韩语。
        FR = "fr"  # 小模型语音识别，法语。
        ID = "id"  # 小模型语音识别，印尼语。
        ES = "es"  # 小模型语音识别，西班牙语。
        PT = "pt"  # 小模型语音识别，葡萄牙语。
        MS = "ms"  # 小模型语音识别，马来语。
        RU = "ru"  # 小模型语音识别，俄语。

    class ASRConfig(BaseModel):
        # 请输入热词列表，以便提升这些词汇的识别准确率。所有热词加起来最多100个 Tokens，超出部分将自动截断。
        hot_words: Optional[List[str]] = None
        # 请输入上下文信息。最多输入 800 个 Tokens，超出部分将自动截断。
        context: Optional[str] = None
        # 请输入上下文信息。最多输入 800 个 Tokens，超出部分将自动截断。
        context_type: Optional[str] = None
        # 用户说话的语种，默认为 common。选项包括：
        user_language: Optional["ChatUpdateEvent.ASRConfigUserLanguage"] = None
        # 将语音转为文本时，是否启用语义顺滑。默认为 true。true：系统在进行语音处理时，会去掉识别结果中诸如 “啊”“嗯” 等语气词，使得输出的文本语义更加流畅自然，符合正常的语言表达习惯，尤其适用于对文本质量要求较高的场景，如正式的会议记录、新闻稿件生成等。false：系统不会对识别结果中的语气词进行处理，识别结果会保留原始的语气词。
        enable_ddc: Optional[bool] = None
        # 将语音转为文本时，是否开启文本规范化（ITN）处理，将识别结果转换为更符合书面表达习惯的格式以提升可读性。默认为 true。开启后，会将口语化数字转换为标准数字格式，示例：将两点十五分转换为 14:15。将一百美元转换为 $100。
        enable_itn: Optional[bool] = None
        # 将语音转为文本时，是否给文本加上标点符号。默认为 true。
        enable_punc: Optional[bool] = None

    class InterruptConfig(BaseModel):
        # 打断模式
        mode: Optional["ChatUpdateEvent.InterruptConfigMode"] = None
        # 打断的关键词配置，最多同时限制 5 个关键词，每个关键词限定长度在6-24个字节以内(2-8个汉字以内), 不能有标点符号。
        keywords: Optional[List[str]] = None

    class TurnDetection(BaseModel):
        # 用户演讲检测模式
        type: Optional["ChatUpdateEvent.TurnDetectionType"] = None
        # server_vad 模式下，VAD 检测到语音之前要包含的音频量，单位为 ms。默认为 600ms。
        prefix_padding_ms: Optional[int] = None
        # server_vad 模式下，检测语音停止的静音持续时间，单位为 ms。默认为 500ms。
        silence_duration_ms: Optional[int] = None
        # server_vad 模式下打断策略配置
        interrupt_config: Optional["ChatUpdateEvent.InterruptConfig"] = None

    class Data(BaseModel):
        # 输出音频格式。
        output_audio: Optional[OutputAudio] = None
        # 输入音频格式。
        input_audio: Optional[InputAudio] = None
        # 对话配置。
        chat_config: Optional["ChatUpdateEvent.ChatConfig"] = None
        # 需要订阅下行事件的事件类型列表。不设置或者设置为空为订阅所有下行事件。
        event_subscriptions: Optional[List[str]] = None
        # 是否需要播放开场白，默认为 false。
        need_play_prologue: Optional[bool] = None
        # 自定义开场白，need_play_prologue 设置为 true 时生效。如果不设定自定义开场白则使用智能体上设置的开场白。
        prologue_content: Optional[str] = None
        # 转检测配置。
        turn_detection: Optional["ChatUpdateEvent.TurnDetection"] = None
        # 语音识别配置，包括热词和上下文信息，以便优化语音识别的准确性和相关性。
        asr_config: Optional["ChatUpdateEvent.ASRConfig"] = None

    event_type: WebsocketsEventType = WebsocketsEventType.CHAT_UPDATE
    data: Data


# req
class ConversationMessageCreateEvent(WebsocketsEvent):
    """手动提交对话内容

    若 role=user，提交事件后就会生成语音回复，适合如下的场景，比如帮我解析 xx 链接，帮我分析这个图片的内容等。若 role=assistant，提交事件后会加入到对话的上下文。
    docs: https://www.coze.cn/open/docs/developer_guides/streaming_chat_event#46f6a7d0
    """

    class Data(BaseModel):
        # 发送这条消息的实体。取值：user（代表该条消息内容是用户发送的）、assistant（代表该条消息内容是智能体发送的）。
        role: str
        # 消息内容的类型，支持设置为：text：文本。object_string：多模态内容，即文本和文件的组合、文本和图片的组合。
        content_type: str
        # 消息的内容，支持纯文本、多模态（文本、图片、文件混合输入）、卡片等多种类型的内容。当 content_type 为 object_string时，content 的结构和详细参数说明请参见object_string object。
        content: str

    event_type: WebsocketsEventType = WebsocketsEventType.CONVERSATION_MESSAGE_CREATE
    data: Data


# req
class ConversationClear(WebsocketsEvent):
    """清除上下文

    清除上下文，会在当前 conversation 下新增一个 section，服务端处理完后会返回 conversation.cleared 事件。
    docs: https://www.coze.cn/open/docs/developer_guides/streaming_chat_event#aa86f213
    """

    event_type: WebsocketsEventType = WebsocketsEventType.CONVERSATION_CLEAR


# req
class ConversationChatSubmitToolOutputsEvent(WebsocketsEvent):
    """提交端插件执行结果

    你可以将需要客户端执行的操作定义为插件，对话中如果触发这个插件，会收到一个 event_type = "conversation.chat.requires_action" 的下行事件，此时需要执行客户端的操作后，通过此上行事件来提交插件执行后的结果。
    docs: https://www.coze.cn/open/docs/developer_guides/streaming_chat_event#aacdcb41
    """

    class Data(BaseModel):
        # 对话的唯一标识。
        chat_id: str
        # 工具执行结果。
        tool_outputs: List[ToolOutput]

    event_type: WebsocketsEventType = WebsocketsEventType.CONVERSATION_CHAT_SUBMIT_TOOL_OUTPUTS
    data: Data


# req
class ConversationChatCancelEvent(WebsocketsEvent):
    """打断智能体输出

    发送此事件可取消正在进行的对话，中断后，服务端将会返回 conversation.chat.canceled 事件。
    docs: https://www.coze.cn/open/docs/developer_guides/streaming_chat_event#0554db7d
    """

    event_type: WebsocketsEventType = WebsocketsEventType.CONVERSATION_CHAT_CANCEL


# resp
class ChatCreatedEvent(WebsocketsEvent):
    """对话连接成功

    流式对话接口成功建立连接后服务端会发送此事件。
    docs: https://www.coze.cn/open/docs/developer_guides/streaming_chat_event#a061f115
    """

    event_type: WebsocketsEventType = WebsocketsEventType.CHAT_CREATED


# resp
class ChatUpdatedEvent(WebsocketsEvent):
    """对话配置成功

    对话配置更新成功后，会返回最新的配置。
    docs: https://www.coze.cn/open/docs/developer_guides/streaming_chat_event#39879618
    """

    event_type: WebsocketsEventType = WebsocketsEventType.CHAT_UPDATED
    data: ChatUpdateEvent.Data


# resp
class ConversationChatCreatedEvent(WebsocketsEvent):
    """对话开始

    创建对话的事件，表示对话开始。
    docs: https://www.coze.cn/open/docs/developer_guides/streaming_chat_event#a2b10fd2
    """

    event_type: WebsocketsEventType = WebsocketsEventType.CONVERSATION_CHAT_CREATED
    data: Chat


# resp
class ConversationChatInProgressEvent(WebsocketsEvent):
    """对话正在处理

    服务端正在处理对话。
    docs: https://www.coze.cn/open/docs/developer_guides/streaming_chat_event#36a38a6b
    """

    event_type: WebsocketsEventType = WebsocketsEventType.CONVERSATION_CHAT_IN_PROGRESS
    data: Chat


# resp
class ConversationMessageDeltaEvent(WebsocketsEvent):
    """增量消息

    增量消息，通常是 type=answer 时的增量消息。
    docs: https://www.coze.cn/open/docs/developer_guides/streaming_chat_event#2dfe8dba
    """

    event_type: WebsocketsEventType = WebsocketsEventType.CONVERSATION_MESSAGE_DELTA
    data: Message


# resp
class ConversationAudioSentenceStartEvent(WebsocketsEvent):
    """增量语音字幕

    一条新的字幕句子，后续的 conversation.audio.delta 增量语音均属于当前字幕句子，可能有多个增量语音共同对应此句字幕的文字内容。
    docs: https://www.coze.cn/open/docs/developer_guides/streaming_chat_event#2e67bf44
    """

    class Data(BaseModel):
        # 新字幕句子的文本内容，后续相关 conversation.audio.delta 增量语音均对应此文本。
        text: str

    event_type: WebsocketsEventType = WebsocketsEventType.CONVERSATION_AUDIO_SENTENCE_START
    data: Data


# resp
class ConversationMessageCompletedEvent(WebsocketsEvent):
    """消息完成

    消息已回复完成。此时事件中带有所有 message.delta 的拼接结果，且每个消息均为 completed 状态。
    docs: https://www.coze.cn/open/docs/developer_guides/streaming_chat_event#4361e8d1
    """

    event_type: WebsocketsEventType = WebsocketsEventType.CONVERSATION_MESSAGE_COMPLETED
    data: Message


# resp
class ConversationAudioCompletedEvent(WebsocketsEvent):
    """语音回复完成

    语音回复完成，表示对话结束。
    docs: https://www.coze.cn/open/docs/developer_guides/streaming_chat_event#b00d6a73
    """

    event_type: WebsocketsEventType = WebsocketsEventType.CONVERSATION_AUDIO_COMPLETED
    data: Message


# resp
class ConversationAudioTranscriptCompletedEvent(WebsocketsEvent):
    """用户语音识别完成

    用户语音识别完成，表示用户语音识别完成。
    docs: https://www.coze.cn/open/docs/developer_guides/streaming_chat_event#9d1e6930
    """

    class Data(BaseModel):
        # 语音识别的最终结果。
        content: str

    event_type: WebsocketsEventType = WebsocketsEventType.CONVERSATION_AUDIO_TRANSCRIPT_COMPLETED
    data: Data


# resp
class ConversationChatRequiresActionEvent(WebsocketsEvent):
    """端插件请求

    对话中断，需要使用方上报工具的执行结果。
    docs: https://www.coze.cn/open/docs/developer_guides/streaming_chat_event#2ef697d8
    """

    event_type: WebsocketsEventType = WebsocketsEventType.CONVERSATION_CHAT_REQUIRES_ACTION
    data: Chat


# resp
class InputAudioBufferSpeechStartedEvent(WebsocketsEvent):
    """用户开始说话

    此事件表示服务端识别到用户正在说话。只有在 server_vad 模式下，才会返回此事件。
    docs: https://www.coze.cn/open/docs/developer_guides/streaming_chat_event#95553c68
    """

    event_type: WebsocketsEventType = WebsocketsEventType.INPUT_AUDIO_BUFFER_SPEECH_STARTED


# resp
class InputAudioBufferSpeechStoppedEvent(WebsocketsEvent):
    """用户结束说话

    此事件表示服务端识别到用户已停止说话。只有在 server_vad 模式下，才会返回此事件。
    docs: https://www.coze.cn/open/docs/developer_guides/streaming_chat_event#5084c0aa
    """

    event_type: WebsocketsEventType = WebsocketsEventType.INPUT_AUDIO_BUFFER_SPEECH_STOPPED


# resp
class ConversationAudioDeltaEvent(WebsocketsEvent):
    """增量语音

    增量消息，通常是 type=answer 时的增量消息。
    docs: https://www.coze.cn/open/docs/developer_guides/streaming_chat_event#36a38a6b
    """

    event_type: WebsocketsEventType = WebsocketsEventType.CONVERSATION_AUDIO_DELTA
    data: Message


# resp
class ConversationChatCompletedEvent(WebsocketsEvent):
    """对话完成

    对话完成，表示对话结束。
    docs: https://www.coze.cn/open/docs/developer_guides/streaming_chat_event#02fac327
    """

    event_type: WebsocketsEventType = WebsocketsEventType.CONVERSATION_CHAT_COMPLETED
    data: Chat


# resp
class ConversationChatFailedEvent(WebsocketsEvent):
    """对话失败

    此事件用于标识对话失败。
    docs: https://www.coze.cn/open/docs/developer_guides/streaming_chat_event#765bb7e5
    """

    event_type: WebsocketsEventType = WebsocketsEventType.CONVERSATION_CHAT_FAILED
    data: Chat


# resp
class ConversationClearedEvent(WebsocketsEvent):
    """上下文清除完成

    上下文清除完成，表示上下文已清除。
    docs: https://www.coze.cn/open/docs/developer_guides/streaming_chat_event#6a941b8a
    """

    event_type: WebsocketsEventType = WebsocketsEventType.CONVERSATION_CLEARED


# resp
class ConversationChatCanceledEvent(WebsocketsEvent):
    """智能体输出中断

    客户端提交 conversation.chat.cancel 事件，服务端完成中断后，将返回此事件。
    docs: https://www.coze.cn/open/docs/developer_guides/streaming_chat_event#089ed144
    """

    class Data(BaseModel):
        # 输出中断类型枚举值，包括 1: 被用户语音说话打断  2: 用户主动 cancel  3: 手动提交对话内容
        code: int
        # 智能体输出中断的详细说明
        msg: str

    event_type: WebsocketsEventType = WebsocketsEventType.CONVERSATION_CHAT_CANCELED
    data: Data


# resp
class ConversationAudioTranscriptUpdateEvent(WebsocketsEvent):
    """用户语音识别字幕

    用户语音识别的中间值，每次返回都是全量文本。
    docs: https://www.coze.cn/open/docs/developer_guides/streaming_chat_event#1b59cbf9
    """

    class Data(BaseModel):
        # 语音识别的中间值。
        content: str

    event_type: WebsocketsEventType = WebsocketsEventType.CONVERSATION_AUDIO_TRANSCRIPT_UPDATE
    data: Data


_chat_event_factory = WebsocketsEventFactory(
    {
        WebsocketsEventType.CHAT_CREATED.value: ChatCreatedEvent,
        WebsocketsEventType.CHAT_UPDATED.value: ChatUpdatedEvent,
        WebsocketsEventType.CONVERSATION_CHAT_CREATED.value: ConversationChatCreatedEvent,
        WebsocketsEventType.CONVERSATION_CHAT_IN_PROGRESS.value: ConversationChatInProgressEvent,
        WebsocketsEventType.CONVERSATION_MESSAGE_DELTA.value: ConversationMessageDeltaEvent,
        WebsocketsEventType.CONVERSATION_AUDIO_SENTENCE_START.value: ConversationAudioSentenceStartEvent,
        WebsocketsEventType.CONVERSATION_AUDIO_DELTA.value: ConversationAudioDeltaEvent,
        WebsocketsEventType.CONVERSATION_MESSAGE_COMPLETED.value: ConversationMessageCompletedEvent,
        WebsocketsEventType.CONVERSATION_AUDIO_COMPLETED.value: ConversationAudioCompletedEvent,
        WebsocketsEventType.CONVERSATION_CHAT_COMPLETED.value: ConversationChatCompletedEvent,
        WebsocketsEventType.CONVERSATION_CHAT_FAILED.value: ConversationChatFailedEvent,
        WebsocketsEventType.INPUT_AUDIO_BUFFER_COMPLETED.value: InputAudioBufferCompletedEvent,
        WebsocketsEventType.INPUT_AUDIO_BUFFER_CLEARED.value: InputAudioBufferClearedEvent,
        WebsocketsEventType.CONVERSATION_CLEARED.value: ConversationClearedEvent,
        WebsocketsEventType.CONVERSATION_CHAT_CANCELED.value: ConversationChatCanceledEvent,
        WebsocketsEventType.CONVERSATION_AUDIO_TRANSCRIPT_UPDATE.value: ConversationAudioTranscriptUpdateEvent,
        WebsocketsEventType.CONVERSATION_AUDIO_TRANSCRIPT_COMPLETED.value: ConversationAudioTranscriptCompletedEvent,
        WebsocketsEventType.CONVERSATION_CHAT_REQUIRES_ACTION.value: ConversationChatRequiresActionEvent,
        WebsocketsEventType.INPUT_AUDIO_BUFFER_SPEECH_STARTED.value: InputAudioBufferSpeechStartedEvent,
        WebsocketsEventType.INPUT_AUDIO_BUFFER_SPEECH_STOPPED.value: InputAudioBufferSpeechStoppedEvent,
    }
)


class WebsocketsChatEventHandler(WebsocketsBaseEventHandler):
    # 对话连接成功
    def on_chat_created(self, cli: "WebsocketsChatClient", event: ChatCreatedEvent):
        pass

    # 对话配置成功
    def on_chat_updated(self, cli: "WebsocketsChatClient", event: ChatUpdatedEvent):
        pass

    # 对话开始
    def on_conversation_chat_created(self, cli: "WebsocketsChatClient", event: ConversationChatCreatedEvent):
        pass

    # 对话正在处理
    def on_conversation_chat_in_progress(self, cli: "WebsocketsChatClient", event: ConversationChatInProgressEvent):
        pass

    # 增量消息
    def on_conversation_message_delta(self, cli: "WebsocketsChatClient", event: ConversationMessageDeltaEvent):
        pass

    # 增量语音字幕
    def on_conversation_audio_sentence_start(
        self, cli: "WebsocketsChatClient", event: ConversationAudioSentenceStartEvent
    ):
        pass

    # 增量语音
    def on_conversation_audio_delta(self, cli: "WebsocketsChatClient", event: ConversationAudioDeltaEvent):
        pass

    # 消息完成
    def on_conversation_message_completed(self, cli: "WebsocketsChatClient", event: ConversationMessageCompletedEvent):
        pass

    # 语音回复完成
    def on_conversation_audio_completed(self, cli: "WebsocketsChatClient", event: ConversationAudioCompletedEvent):
        pass

    # 对话完成
    def on_conversation_chat_completed(self, cli: "WebsocketsChatClient", event: ConversationChatCompletedEvent):
        pass

    # 对话失败
    def on_conversation_chat_failed(self, cli: "WebsocketsChatClient", event: ConversationChatFailedEvent):
        pass

    # 音频提交完成
    def on_input_audio_buffer_completed(self, cli: "WebsocketsChatClient", event: InputAudioBufferCompletedEvent):
        pass

    # 音频清除成功
    def on_input_audio_buffer_cleared(self, cli: "WebsocketsChatClient", event: InputAudioBufferClearedEvent):
        pass

    # 上下文清除完成
    def on_conversation_cleared(self, cli: "WebsocketsChatClient", event: ConversationClearedEvent):
        pass

    # 智能体输出中断
    def on_conversation_chat_canceled(self, cli: "WebsocketsChatClient", event: ConversationChatCanceledEvent):
        pass

    # 用户语音识别字幕
    def on_conversation_audio_transcript_update(
        self, cli: "WebsocketsChatClient", event: ConversationAudioTranscriptUpdateEvent
    ):
        pass

    # 用户语音识别完成
    def on_conversation_audio_transcript_completed(
        self, cli: "WebsocketsChatClient", event: ConversationAudioTranscriptCompletedEvent
    ):
        pass

    # 端插件请求
    def on_conversation_chat_requires_action(
        self, cli: "WebsocketsChatClient", event: ConversationChatRequiresActionEvent
    ):
        pass

    # 用户开始说话
    def on_input_audio_buffer_speech_started(
        self, cli: "WebsocketsChatClient", event: InputAudioBufferSpeechStartedEvent
    ):
        pass

    # 用户结束说话
    def on_input_audio_buffer_speech_stopped(
        self, cli: "WebsocketsChatClient", event: InputAudioBufferSpeechStoppedEvent
    ):
        pass


class WebsocketsChatClient(WebsocketsBaseClient):
    def __init__(
        self,
        base_url: str,
        requester: Requester,
        bot_id: str,
        on_event: Union[WebsocketsChatEventHandler, Dict[WebsocketsEventType, Callable]],
        workflow_id: Optional[str] = None,
        device_id: Optional[int] = None,
        **kwargs,
    ):
        if isinstance(on_event, WebsocketsChatEventHandler):
            on_event = on_event.to_dict()
        super().__init__(
            base_url=base_url,
            requester=requester,
            path="v1/chat",
            event_factory=_chat_event_factory,
            query=remove_none_values(
                {
                    "bot_id": bot_id,
                    "workflow_id": workflow_id,
                    "device_id": str(device_id) if device_id is not None else None,
                }
            ),
            on_event=on_event,  # type: ignore
            wait_events=[WebsocketsEventType.CONVERSATION_CHAT_COMPLETED],
            **kwargs,
        )

    # 更新对话配置
    def chat_update(self, data: ChatUpdateEvent.Data) -> None:
        self._input_queue.put(ChatUpdateEvent.model_validate({"data": data}))

    # 流式上传音频片段
    def input_audio_buffer_append(self, data: InputAudioBufferAppendEvent.Data) -> None:
        self._input_queue.put(InputAudioBufferAppendEvent.model_validate({"data": data}))

    # 提交音频
    def input_audio_buffer_complete(self) -> None:
        self._input_queue.put(InputAudioBufferCompleteEvent.model_validate({}))

    # 清除缓冲区音频
    def input_audio_buffer_clear(self) -> None:
        self._input_queue.put(InputAudioBufferClearEvent.model_validate({}))

    # 手动提交对话内容
    def conversation_chat_submit_tool_outputs(self, data: ConversationChatSubmitToolOutputsEvent.Data) -> None:
        self._input_queue.put(ConversationChatSubmitToolOutputsEvent.model_validate({"data": data}))

    # 清除上下文
    def conversation_clear(self) -> None:
        self._input_queue.put(ConversationClear.model_validate({}))

    # 提交端插件执行结果
    def conversation_chat_cancel(self) -> None:
        self._input_queue.put(ConversationChatCancelEvent.model_validate({}))

    # 打断智能体输出
    def conversation_message_create(self, data: ConversationMessageCreateEvent.Data) -> None:
        self._input_queue.put(ConversationMessageCreateEvent.model_validate({"data": data}))


class WebsocketsChatBuildClient(object):
    def __init__(self, base_url: str, requester: Requester):
        self._base_url = remove_url_trailing_slash(base_url)
        self._requester = requester

    def create(
        self,
        *,
        bot_id: str,
        on_event: Union[WebsocketsChatEventHandler, Dict[WebsocketsEventType, Callable]],
        workflow_id: Optional[str] = None,
        device_id: Optional[int] = None,
        **kwargs,
    ) -> WebsocketsChatClient:
        return WebsocketsChatClient(
            base_url=self._base_url,
            requester=self._requester,
            bot_id=bot_id,
            on_event=on_event,  # type: ignore
            workflow_id=workflow_id,
            device_id=device_id,
            **kwargs,
        )


class AsyncWebsocketsChatEventHandler(AsyncWebsocketsBaseEventHandler):
    # 对话连接成功
    async def on_chat_created(self, cli: "AsyncWebsocketsChatClient", event: ChatCreatedEvent):
        pass

    # 对话配置成功
    async def on_chat_updated(self, cli: "AsyncWebsocketsChatClient", event: ChatUpdatedEvent):
        pass

    # 对话开始
    async def on_conversation_chat_created(self, cli: "AsyncWebsocketsChatClient", event: ConversationChatCreatedEvent):
        pass

    # 对话正在处理
    async def on_conversation_chat_in_progress(
        self, cli: "AsyncWebsocketsChatClient", event: ConversationChatInProgressEvent
    ):
        pass

    # 增量消息
    async def on_conversation_message_delta(
        self, cli: "AsyncWebsocketsChatClient", event: ConversationMessageDeltaEvent
    ):
        pass

    # 增量语音字幕
    async def on_conversation_audio_sentence_start(
        self, cli: "AsyncWebsocketsChatClient", event: ConversationAudioSentenceStartEvent
    ):
        pass

    # 增量语音
    async def on_conversation_audio_delta(self, cli: "AsyncWebsocketsChatClient", event: ConversationAudioDeltaEvent):
        pass

    # 消息完成
    async def on_conversation_message_completed(
        self, cli: "AsyncWebsocketsChatClient", event: ConversationMessageCompletedEvent
    ):
        pass

    # 语音回复完成
    async def on_conversation_audio_completed(
        self, cli: "AsyncWebsocketsChatClient", event: ConversationAudioCompletedEvent
    ):
        pass

    # 对话完成
    async def on_conversation_chat_completed(
        self, cli: "AsyncWebsocketsChatClient", event: ConversationChatCompletedEvent
    ):
        pass

    # 对话失败
    async def on_conversation_chat_failed(self, cli: "AsyncWebsocketsChatClient", event: ConversationChatFailedEvent):
        pass

    # 音频提交完成
    async def on_input_audio_buffer_completed(
        self, cli: "AsyncWebsocketsChatClient", event: InputAudioBufferCompletedEvent
    ):
        pass

    # 音频清除成功
    async def on_input_audio_buffer_cleared(
        self, cli: "AsyncWebsocketsChatClient", event: InputAudioBufferClearedEvent
    ):
        pass

    # 上下文清除完成
    async def on_conversation_cleared(self, cli: "AsyncWebsocketsChatClient", event: ConversationClearedEvent):
        pass

    # 智能体输出中断
    async def on_conversation_chat_canceled(
        self, cli: "AsyncWebsocketsChatClient", event: ConversationChatCanceledEvent
    ):
        pass

    # 用户语音识别字幕
    async def on_conversation_audio_transcript_update(
        self, cli: "AsyncWebsocketsChatClient", event: ConversationAudioTranscriptUpdateEvent
    ):
        pass

    # 用户语音识别完成
    async def on_conversation_audio_transcript_completed(
        self, cli: "AsyncWebsocketsChatClient", event: ConversationAudioTranscriptCompletedEvent
    ):
        pass

    # 端插件请求
    async def on_conversation_chat_requires_action(
        self, cli: "AsyncWebsocketsChatClient", event: ConversationChatRequiresActionEvent
    ):
        pass

    # 用户开始说话
    async def on_input_audio_buffer_speech_started(
        self, cli: "AsyncWebsocketsChatClient", event: InputAudioBufferSpeechStartedEvent
    ):
        pass

    # 用户结束说话
    async def on_input_audio_buffer_speech_stopped(
        self, cli: "AsyncWebsocketsChatClient", event: InputAudioBufferSpeechStoppedEvent
    ):
        pass


class AsyncWebsocketsChatClient(AsyncWebsocketsBaseClient):
    def __init__(
        self,
        base_url: str,
        requester: Requester,
        bot_id: str,
        on_event: Union[AsyncWebsocketsChatEventHandler, Dict[WebsocketsEventType, Callable]],
        workflow_id: Optional[str] = None,
        device_id: Optional[int] = None,
        **kwargs,
    ):
        if isinstance(on_event, AsyncWebsocketsChatEventHandler):
            on_event = on_event.to_dict()
        super().__init__(
            base_url=base_url,
            requester=requester,
            path="v1/chat",
            event_factory=_chat_event_factory,
            query=remove_none_values(
                {
                    "bot_id": bot_id,
                    "workflow_id": workflow_id,
                    "device_id": str(device_id) if device_id is not None else None,
                }
            ),
            on_event=on_event,  # type: ignore
            wait_events=[WebsocketsEventType.CONVERSATION_CHAT_COMPLETED],
            **kwargs,
        )

    # 更新对话配置
    async def chat_update(self, data: ChatUpdateEvent.Data) -> None:
        await self._input_queue.put(ChatUpdateEvent.model_validate({"data": data}))

    # 流式上传音频片段
    async def input_audio_buffer_append(self, data: InputAudioBufferAppendEvent.Data) -> None:
        await self._input_queue.put(InputAudioBufferAppendEvent.model_validate({"data": data}))

    # 提交音频
    async def input_audio_buffer_complete(self) -> None:
        await self._input_queue.put(InputAudioBufferCompleteEvent.model_validate({}))

    # 清除缓冲区音频
    async def input_audio_buffer_clear(self) -> None:
        await self._input_queue.put(InputAudioBufferClearEvent.model_validate({}))

    # 手动提交对话内容
    async def conversation_chat_submit_tool_outputs(self, data: ConversationChatSubmitToolOutputsEvent.Data) -> None:
        await self._input_queue.put(ConversationChatSubmitToolOutputsEvent.model_validate({"data": data}))

    # 清除上下文
    async def conversation_clear(self) -> None:
        await self._input_queue.put(ConversationClear.model_validate({}))

    # 提交端插件执行结果
    async def conversation_chat_cancel(self) -> None:
        await self._input_queue.put(ConversationChatCancelEvent.model_validate({}))

    # 打断智能体输出
    async def conversation_message_create(self, data: ConversationMessageCreateEvent.Data) -> None:
        await self._input_queue.put(ConversationMessageCreateEvent.model_validate({"data": data}))


class AsyncWebsocketsChatBuildClient(object):
    def __init__(self, base_url: str, requester: Requester):
        self._base_url = remove_url_trailing_slash(base_url)
        self._requester = requester

    def create(
        self,
        *,
        bot_id: str,
        on_event: Union[AsyncWebsocketsChatEventHandler, Dict[WebsocketsEventType, Callable]],
        workflow_id: Optional[str] = None,
        device_id: Optional[int] = None,
        **kwargs,
    ) -> AsyncWebsocketsChatClient:
        return AsyncWebsocketsChatClient(
            base_url=self._base_url,
            requester=self._requester,
            bot_id=bot_id,
            on_event=on_event,  # type: ignore
            workflow_id=workflow_id,
            device_id=device_id,
            **kwargs,
        )



from typing import TYPE_CHECKING, List, Optional

from cozepy.bots import PublishStatus
from cozepy.model import AsyncNumberPaged, CozeModel, DynamicStrEnum, HTTPRequest, NumberPaged, NumberPagedResponse
from cozepy.request import Requester
from cozepy.util import remove_none_values, remove_url_trailing_slash

if TYPE_CHECKING:
    from .chat import AsyncWorkflowsChatClient, WorkflowsChatClient
    from .runs import AsyncWorkflowsRunsClient, WorkflowsRunsClient


class WorkflowMode(DynamicStrEnum):
    WORKFLOW = "workflow"
    CHATFLOW = "chatflow"


class WorkflowInfo(CozeModel):
    workflow_id: str
    workflow_name: str
    description: str
    icon_url: str
    app_id: str


class _PrivateListWorkflowData(CozeModel, NumberPagedResponse[WorkflowInfo]):
    items: List[WorkflowInfo]
    has_more: bool

    def get_total(self) -> Optional[int]:
        return None

    def get_has_more(self) -> Optional[bool]:
        return self.has_more

    def get_items(self) -> List[WorkflowInfo]:
        return self.items


class WorkflowsClient(object):
    def __init__(self, base_url: str, requester: Requester):
        self._base_url = remove_url_trailing_slash(base_url)
        self._requester = requester
        self._runs: Optional[WorkflowsRunsClient] = None
        self._chat: Optional[WorkflowsChatClient] = None

    @property
    def runs(self) -> "WorkflowsRunsClient":
        if not self._runs:
            from .runs import WorkflowsRunsClient

            self._runs = WorkflowsRunsClient(self._base_url, self._requester)
        return self._runs

    @property
    def chat(self) -> "WorkflowsChatClient":
        if not self._chat:
            from .chat import WorkflowsChatClient

            self._chat = WorkflowsChatClient(self._base_url, self._requester)
        return self._chat

    def list(
        self,
        *,
        workspace_id: Optional[str] = None,
        workflow_mode: Optional[WorkflowMode] = None,
        app_id: Optional[str] = None,
        publish_status: Optional[PublishStatus] = None,
        page_num: int = 1,
        page_size: int = 100,
    ) -> NumberPaged[WorkflowInfo]:
        url = f"{self._base_url}/v1/workflows"

        def request_maker(i_page_num: int, i_page_size: int) -> HTTPRequest:
            return self._requester.make_request(
                "GET",
                url,
                params=remove_none_values(
                    {
                        "workspace_id": workspace_id,
                        "workflow_mode": workflow_mode,
                        "app_id": app_id,
                        "publish_status": publish_status,
                        "page_num": i_page_num,
                        "page_size": i_page_size,
                    }
                ),
                cast=_PrivateListWorkflowData,
                stream=False,
            )

        return NumberPaged(
            page_num=page_num,
            page_size=page_size,
            requestor=self._requester,
            request_maker=request_maker,
        )


class AsyncWorkflowsClient(object):
    def __init__(self, base_url: str, requester: Requester):
        self._base_url = remove_url_trailing_slash(base_url)
        self._requester = requester
        self._runs: Optional[AsyncWorkflowsRunsClient] = None
        self._chat: Optional[AsyncWorkflowsChatClient] = None

    @property
    def runs(self) -> "AsyncWorkflowsRunsClient":
        if not self._runs:
            from .runs import AsyncWorkflowsRunsClient

            self._runs = AsyncWorkflowsRunsClient(self._base_url, self._requester)
        return self._runs

    @property
    def chat(self) -> "AsyncWorkflowsChatClient":
        if not self._chat:
            from .chat import AsyncWorkflowsChatClient

            self._chat = AsyncWorkflowsChatClient(self._base_url, self._requester)
        return self._chat

    async def list(
        self,
        *,
        workspace_id: Optional[str] = None,
        workflow_mode: Optional[WorkflowMode] = None,
        app_id: Optional[str] = None,
        publish_status: Optional[PublishStatus] = None,
        page_num: int = 1,
        page_size: int = 100,
        **kwargs,
    ) -> AsyncNumberPaged[WorkflowInfo]:
        url = f"{self._base_url}/v1/workflows"
        headers: Optional[dict] = kwargs.get("headers")

        async def request_maker(i_page_num: int, i_page_size: int) -> HTTPRequest:
            return await self._requester.amake_request(
                "GET",
                url,
                params=remove_none_values(
                    {
                        "workspace_id": workspace_id,
                        "workflow_mode": workflow_mode,
                        "app_id": app_id,
                        "publish_status": publish_status,
                        "page_num": i_page_num,
                        "page_size": i_page_size,
                    }
                ),
                headers=headers,
                cast=_PrivateListWorkflowData,
                stream=False,
            )

        return await AsyncNumberPaged.build(
            page_num=page_num,
            page_size=page_size,
            requestor=self._requester,
            request_maker=request_maker,
        )



from typing import Optional

from cozepy.model import CozeModel
from cozepy.request import Requester
from cozepy.util import remove_url_trailing_slash


class WorkflowNodeExecuteHistory(CozeModel):
    # 节点是否执行完成。true 表示执行已完成，false表示执行未完成。
    is_finish: bool
    # 节点的执行结果输出信息。
    node_output: Optional[str] = None


class WorkflowsRunsRunHistoriesExecuteNodesClient(object):
    def __init__(self, base_url: str, requester: Requester):
        self._base_url = remove_url_trailing_slash(base_url)
        self._requester = requester

    def retrieve(
        self, *, workflow_id: str, execute_id: str, node_execute_uuid: str, **kwargs
    ) -> WorkflowNodeExecuteHistory:
        """查询工作流节点的运行结果

        :param workflow_id: 工作流 ID。
        :param execute_id: 工作流执行 ID。
        :param node_execute_uuid: 工作流异步运行结果 API 中返回的节点执行 uuid。
        """
        url = (
            f"{self._base_url}/v1/workflows/{workflow_id}/run_histories/{execute_id}/execute_nodes/{node_execute_uuid}"
        )
        headers: Optional[dict] = kwargs.get("headers")
        return self._requester.request("get", url, False, cast=WorkflowNodeExecuteHistory, headers=headers)


class AsyncWorkflowsRunsRunHistoriesExecuteNodesClient(object):
    def __init__(self, base_url: str, requester: Requester):
        self._base_url = remove_url_trailing_slash(base_url)
        self._requester = requester

    async def retrieve(
        self, *, workflow_id: str, execute_id: str, node_execute_uuid: str, **kwargs
    ) -> WorkflowNodeExecuteHistory:
        """查询工作流节点的运行结果

        :param workflow_id: 工作流 ID。
        :param execute_id: 工作流执行 ID。
        :param node_execute_uuid: 工作流异步运行结果 API 中返回的节点执行 uuid。
        """
        url = (
            f"{self._base_url}/v1/workflows/{workflow_id}/run_histories/{execute_id}/execute_nodes/{node_execute_uuid}"
        )
        headers: Optional[dict] = kwargs.get("headers")
        return await self._requester.arequest("get", url, False, cast=WorkflowNodeExecuteHistory, headers=headers)



import abc
import asyncio
import json
import logging
import queue
import sys
import threading
import traceback
from abc import ABC
from contextlib import asynccontextmanager, contextmanager
from functools import lru_cache
from typing import Any, Callable, Dict, List, Optional, Set, Tuple, Type, Union, get_type_hints

if sys.version_info >= (3, 8):
    # note: >=3.7,<3.8 not support asyncio
    from websockets import InvalidStatus
    from websockets.asyncio.client import ClientConnection as AsyncWebsocketClientConnection
    from websockets.asyncio.client import connect as asyncio_connect
else:
    # 警告: 当前Python版本不支持asyncio websockets
    # 如果Python版本小于3.8,则不支持异步websockets功能
    import warnings

    warnings.warn("asyncio websockets requires Python >= 3.8")

    class AsyncWebsocketClientConnection(object):
        def recv(self, *args, **kwargs):
            pass

        def send(self, *args, **kwargs):
            pass

        def close(self, *args, **kwargs):
            pass

    def asyncio_connect(*args, **kwargs):
        pass

    class InvalidStatus(object):
        pass


import websockets.sync.client
from pydantic import BaseModel

from cozepy.exception import CozeAPIError
from cozepy.log import log_debug, log_error, log_info, log_warning, logger
from cozepy.model import CozeModel, DynamicStrEnum
from cozepy.request import Requester
from cozepy.util import get_methods, get_model_default, remove_url_trailing_slash
from cozepy.version import coze_client_user_agent, user_agent


class WebsocketsEventType(DynamicStrEnum):
    # common
    CLIENT_ERROR = "client_error"  # sdk error
    CLOSED = "closed"  # connection closed

    # error
    ERROR = "error"  # 发生错误

    # v1/audio/speech
    # req
    SPEECH_UPDATE = "speech.update"  # 更新语音合成配置
    INPUT_TEXT_BUFFER_APPEND = "input_text_buffer.append"  # 流式输入文字
    INPUT_TEXT_BUFFER_COMPLETE = "input_text_buffer.complete"  # 提交文字
    # resp
    # v1/audio/speech
    SPEECH_CREATED = "speech.created"  # 语音合成连接成功
    SPEECH_UPDATED = "speech.updated"  # 配置更新完成
    INPUT_TEXT_BUFFER_COMPLETED = "input_text_buffer.completed"  # input_text_buffer 提交完成
    SPEECH_AUDIO_UPDATE = "speech.audio.update"  # 合成增量语音
    SPEECH_AUDIO_COMPLETED = "speech.audio.completed"  # 合成完成

    # v1/audio/transcriptions
    # req
    TRANSCRIPTIONS_UPDATE = "transcriptions.update"  # 更新语音识别配置
    INPUT_AUDIO_BUFFER_APPEND = "input_audio_buffer.append"  # 流式上传音频片段
    INPUT_AUDIO_BUFFER_COMPLETE = "input_audio_buffer.complete"  # 提交音频
    INPUT_AUDIO_BUFFER_CLEAR = "input_audio_buffer.clear"  # 清除缓冲区音频
    # resp
    TRANSCRIPTIONS_CREATED = "transcriptions.created"  # 连接成功
    TRANSCRIPTIONS_UPDATED = "transcriptions.updated"  # 配置更新成功
    INPUT_AUDIO_BUFFER_COMPLETED = "input_audio_buffer.completed"  # 音频提交完成
    INPUT_AUDIO_BUFFER_CLEARED = "input_audio_buffer.cleared"  # 音频清除成功
    TRANSCRIPTIONS_MESSAGE_UPDATE = "transcriptions.message.update"  # 识别出文字
    TRANSCRIPTIONS_MESSAGE_COMPLETED = "transcriptions.message.completed"  # 识别完成

    # v1/chat
    # req
    CHAT_UPDATE = "chat.update"  # 更新对话配置
    # INPUT_AUDIO_BUFFER_APPEND = "input_audio_buffer.append"  # 流式上传音频片段
    # INPUT_AUDIO_BUFFER_COMPLETE = "input_audio_buffer.complete"  # 提交音频
    # INPUT_AUDIO_BUFFER_CLEAR = "input_audio_buffer.clear"  # 清除缓冲区音频
    CONVERSATION_MESSAGE_CREATE = "conversation.message.create"  # 手动提交对话内容
    CONVERSATION_CLEAR = "conversation.clear"  # 清除上下文
    CONVERSATION_CHAT_SUBMIT_TOOL_OUTPUTS = "conversation.chat.submit_tool_outputs"  # 提交端插件执行结果
    CONVERSATION_CHAT_CANCEL = "conversation.chat.cancel"  # 打断智能体输出
    # resp
    CHAT_CREATED = "chat.created"  # 对话连接成功
    CHAT_UPDATED = "chat.updated"  # 对话配置成功
    CONVERSATION_CHAT_CREATED = "conversation.chat.created"  # 对话开始
    CONVERSATION_CHAT_IN_PROGRESS = "conversation.chat.in_progress"  # 对话正在处理
    CONVERSATION_MESSAGE_DELTA = "conversation.message.delta"  # 增量消息
    CONVERSATION_AUDIO_SENTENCE_START = "conversation.audio.sentence_start"  # 增量语音字幕
    CONVERSATION_AUDIO_DELTA = "conversation.audio.delta"  # 增量语音
    CONVERSATION_MESSAGE_COMPLETED = "conversation.message.completed"  # 消息完成
    CONVERSATION_AUDIO_COMPLETED = "conversation.audio.completed"  # 语音回复完成
    CONVERSATION_CHAT_COMPLETED = "conversation.chat.completed"  # 对话完成
    CONVERSATION_CHAT_FAILED = "conversation.chat.failed"  # 对话失败
    # INPUT_AUDIO_BUFFER_COMPLETED = "input_audio_buffer.completed"  # 音频提交完成
    # INPUT_AUDIO_BUFFER_CLEARED = "input_audio_buffer.cleared"  # 音频清除成功
    CONVERSATION_CLEARED = "conversation.cleared"  # 上下文清除完成
    CONVERSATION_CHAT_CANCELED = "conversation.chat.canceled"  # 智能体输出中断
    CONVERSATION_AUDIO_TRANSCRIPT_UPDATE = "conversation.audio_transcript.update"  # 用户语音识别字幕
    CONVERSATION_AUDIO_TRANSCRIPT_COMPLETED = "conversation.audio_transcript.completed"  # 用户语音识别完成
    CONVERSATION_CHAT_REQUIRES_ACTION = "conversation.chat.requires_action"  # 端插件请求
    INPUT_AUDIO_BUFFER_SPEECH_STARTED = "input_audio_buffer.speech_started"  # 用户开始说话
    INPUT_AUDIO_BUFFER_SPEECH_STOPPED = "input_audio_buffer.speech_stopped"  # 用户结束说话


class WebsocketsEvent(CozeModel, ABC):
    class Detail(BaseModel):
        logid: Optional[str] = None
        # if event_type=error, origin_message is the original input message
        origin_message: Optional[str] = None

    event_type: WebsocketsEventType
    id: Optional[str] = None
    detail: Optional[Detail] = None


class WebsocketsErrorEvent(WebsocketsEvent):
    """发生错误"""

    event_type: WebsocketsEventType = WebsocketsEventType.ERROR
    data: CozeAPIError


class LimitConfig(BaseModel):
    # 周期的时长，单位为秒。例如设置为 10 秒，则以 10 秒作为一个周期。
    period: Optional[int] = None
    # 周期内，最大返回包数量。
    max_frame_num: Optional[int] = None


class InputAudio(BaseModel):
    # 输入音频的格式，支持 pcm、wav、ogg。默认为 wav。
    format: Optional[str]
    # 输入音频的编码，支持 pcm、opus、g711a、g711u。默认为 pcm。如果音频编码格式为 g711a 或 g711u，format 请设置为 pcm。
    codec: Optional[str]
    # 输入音频的采样率，默认是 24000。支持 8000、16000、22050、24000、32000、44100、48000。如果音频编码格式 codec 为 g711a 或 g711u，音频采样率需设置为 8000。
    sample_rate: Optional[int]
    # 输入音频的声道数，支持 1（单声道）、2（双声道）。默认是 1（单声道）。
    channel: Optional[int]
    # 输入音频的位深，默认是 16，支持8、16和24。
    bit_depth: Optional[int]


class OpusConfig(BaseModel):
    # 输出 opus 的码率，默认 48000。
    bitrate: Optional[int] = None
    # 输出 opus 是否使用 CBR 编码，默认为 false。
    use_cbr: Optional[bool] = None
    # 输出 opus 的帧长，默认是 10。可选值：2.5、5、10、20、40、60
    frame_size_ms: Optional[float] = None
    # 输出音频限流配置，默认不限制。
    limit_config: Optional[LimitConfig] = None


class PCMConfig(BaseModel):
    # 输出 pcm 音频的采样率，默认是 24000。支持 8000、16000、22050、24000、32000、44100、48000。
    sample_rate: Optional[int] = None
    # 输出每个 pcm 包的时长，单位 ms，默认不限制。
    frame_size_ms: Optional[float] = None
    # 输出音频限流配置，默认不限制。
    limit_config: Optional[LimitConfig] = None


class OutputAudio(BaseModel):
    # 输出音频编码，支持 pcm、g711a、g711u、opus。默认是 pcm。当 codec 设置为 pcm、g711a或 g711u时，你可以通过 pcm_config 配置 PCM 音频参数。
    codec: Optional[str]
    # 当 codec 设置为 pcm、g711a 或 g711u 时，用于配置 PCM 音频参数。当 codec 设置为 opus 时，不需要设置此字段
    pcm_config: Optional[PCMConfig] = None
    # 当 codec 设置为 pcm 时，不需要设置此字段。
    opus_config: Optional[OpusConfig] = None
    # 输出音频的语速，取值范围 [-50, 100]，默认为 0。-50 表示 0.5 倍速，100 表示 2 倍速。
    speech_rate: Optional[int] = None
    # 输出音频的音色 ID，默认是柔美女友音色。你可以调用[查看音色列表](https://www.coze.cn/open/docs/developer_guides/list_voices) API 查看当前可用的所有音色 ID。
    voice_id: Optional[str] = None


class WebsocketsEventFactory(object):
    def __init__(self, event_type_to_class: Dict[str, Type[WebsocketsEvent]]):
        self._event_type_to_class = event_type_to_class

    @lru_cache(maxsize=128)
    def get_event_class(
        self,
        event_type: str,
    ) -> Tuple[Optional[Type[WebsocketsEvent]], Optional[Type[BaseModel]]]:
        event_class = self._event_type_to_class.get(event_type)
        if not event_class:
            return None, None

        type_hints = get_type_hints(event_class)
        data_type = type_hints.get("data")
        if data_type:
            return event_class, data_type
        return event_class, None

    def create_event(self, path: str, message: Dict) -> Optional[WebsocketsEvent]:
        event_id = message.get("id") or ""
        detail = WebsocketsEvent.Detail.model_validate(message.get("detail") or {})
        event_type = message.get("event_type") or ""
        data = message.get("data") or {}

        event_class, data_class = self.get_event_class(event_type)
        if not event_class:
            log_warning("[%s] unknown event, type=%s, logid=%s", path, event_type, detail.logid)
            return None

        event_data = {
            "id": event_id,
            "detail": detail,
        }

        if data and data_class:
            event_data["data"] = data_class.model_validate(data)
        return event_class.model_validate(event_data)


def _log_receive_event(path: str, event_type: Optional[str], data: Union[str, bytes]):
    if logger.level > logging.DEBUG:
        return
    if event_type and event_type == "conversation.audio.delta":
        message = json.loads(data)
        if message and message.get("data") and message.get("data", {}).get("content"):
            message["data"]["content"] = f"<length: {len(message['data']['content'])}>"
            log_debug("[%s] receive event, type=%s, event=%s", path, event_type, json.dumps(message))
            return
    log_debug("[%s] receive event, type=%s, event=%s", path, event_type, data)


def _log_send_event(path: str, event: WebsocketsEvent):
    if logger.level > logging.DEBUG:
        return
    if event.event_type == WebsocketsEventType.INPUT_AUDIO_BUFFER_APPEND:
        log_debug(
            "[%s] send event, type=%s, event=%s",
            path,
            event.event_type.value,
            json.dumps(event._dump_without_delta()),  # type: ignore
        )
    else:
        log_debug(
            "[%s] send event, type=%s, event=%s",
            path,
            event.event_type.value,
            event.model_dump_json(exclude_none=True),
        )


class WebsocketsBaseClient(abc.ABC):
    class State(DynamicStrEnum):
        """
        initialized, connecting, connected, closing, closed
        """

        INITIALIZED = "initialized"
        CONNECTING = "connecting"
        CONNECTED = "connected"
        CLOSING = "closing"
        CLOSED = "closed"

    def __init__(
        self,
        base_url: str,
        requester: Requester,
        path: str,
        event_factory: WebsocketsEventFactory,
        query: Optional[Dict[str, str]] = None,
        on_event: Optional[Dict[WebsocketsEventType, Callable]] = None,
        wait_events: Optional[List[WebsocketsEventType]] = None,
        **kwargs,
    ):
        self._state = self.State.INITIALIZED
        self._base_url = remove_url_trailing_slash(base_url)
        self._requester = requester
        self._path = path
        self._ws_url = self._base_url + "/" + path
        if query:
            self._ws_url += "?" + "&".join([f"{k}={v}" for k, v in query.items()])
        self._on_event = on_event.copy() if on_event else {}
        self._headers = kwargs.get("headers")
        self._wait_events = wait_events.copy() if wait_events else []
        self._event_factory = event_factory

        self._input_queue: queue.Queue[Optional[WebsocketsEvent]] = queue.Queue()
        self._ws: Optional[websockets.sync.client.ClientConnection] = None
        self._send_thread: Optional[threading.Thread] = None
        self._receive_thread: Optional[threading.Thread] = None
        self._completed_events: Set[WebsocketsEventType] = set()
        self._completed_event = threading.Event()
        self._join_event = threading.Event()

    @contextmanager
    def __call__(self):
        try:
            self.connect()
            yield self
        finally:
            self.close()

    def connect(self):
        if self._state != self.State.INITIALIZED:
            raise ValueError(f"Cannot connect in {self._state.value} state")
        self._state = self.State.CONNECTING
        headers = {
            "X-Coze-Client-User-Agent": coze_client_user_agent(),
            **(self._headers or {}),
        }

        self._requester.auth_header(headers)

        try:
            self._ws = websockets.sync.client.connect(
                self._ws_url,
                user_agent_header=user_agent(),
                additional_headers=headers,
            )
            self._state = self.State.CONNECTED
            log_info("[%s] connected to websocket", self._path)

            self._send_thread = threading.Thread(target=self._send_loop)
            self._receive_thread = threading.Thread(target=self._receive_loop)
            self._send_thread.start()
            self._receive_thread.start()
        except InvalidStatus as e:
            raise CozeAPIError(None, f"{e}", e.response.headers.get("x-tt-logid")) from e

    def wait(self, events: Optional[List[WebsocketsEventType]] = None, wait_all=True) -> None:
        if events is None:
            events = self._wait_events
        self._wait_completed(events, wait_all=wait_all)

    def on(self, event_type: WebsocketsEventType, handler: Callable):
        self._on_event[event_type] = handler

    def close(self) -> None:
        if self._state not in (self.State.CONNECTED, self.State.CONNECTING):
            return
        self._state = self.State.CLOSING
        self._join_event.set()
        self._close()
        self._state = self.State.CLOSED

    def _send_loop(self) -> None:
        try:
            while not self._join_event.is_set():
                try:
                    event = self._input_queue.get(timeout=0.5)
                    self._send_event(event)
                    self._input_queue.task_done()
                except queue.Empty:
                    pass
        except Exception as e:
            self._handle_error(e)

    def _receive_loop(self) -> None:
        try:
            while not self._join_event.is_set():
                if not self._ws:
                    log_debug("[%s] empty websocket conn, close", self._path)
                    break

                try:
                    data = self._ws.recv(timeout=0.5)
                    message = json.loads(data)
                    event_type = message.get("event_type")
                    _log_receive_event(self._path, event_type, data)

                    event = self._parse_event(message)
                    if event:
                        handler = self._on_event.get(event_type)
                        if handler:
                            handler(self, event)
                        self._completed_events.add(event_type)
                        self._completed_event.set()
                except TimeoutError:
                    pass
        except Exception as e:
            self._handle_error(e)

    def _parse_event(self, message: Dict) -> Optional[WebsocketsEvent]:
        event_id = message.get("id") or ""
        event_type = message.get("event_type") or ""
        detail = WebsocketsEvent.Detail.model_validate(message.get("detail") or {})
        data = message.get("data") or {}
        if event_type == WebsocketsEventType.ERROR.value:
            code, msg = data.get("code") or 0, data.get("msg") or ""
            return WebsocketsErrorEvent.model_validate(
                {
                    "id": event_id,
                    "detail": detail,
                    "data": CozeAPIError(code, msg, WebsocketsEvent.Detail.model_validate(detail).logid),
                }
            )
        return self._event_factory.create_event(self._path, message)

    def _wait_completed(self, events: List[WebsocketsEventType], wait_all: bool) -> None:
        while True:
            if wait_all:
                # 所有事件都需要完成
                if self._completed_events == set(events):
                    break
            else:
                # 任意一个事件完成即可
                if any(event in self._completed_events for event in events):
                    break
            self._completed_event.wait()
            self._completed_event.clear()

    def _handle_error(self, error: Exception) -> None:
        handler = self._on_event.get(WebsocketsEventType.ERROR)
        if handler:
            handler(self, error)
        else:
            raise error

    def _close(self) -> None:
        log_info("[%s] connect closed", self._path)
        if self._send_thread:
            self._send_thread.join()
        if self._receive_thread:
            self._receive_thread.join()

        if self._ws:
            self._ws.close()
            self._ws = None

        while not self._input_queue.empty():
            self._input_queue.get()

        handler = self._on_event.get(WebsocketsEventType.CLOSED)
        if handler:
            handler(self)

    def _send_event(self, event: Optional[WebsocketsEvent] = None) -> None:
        if not event or not self._ws:
            return
        log_debug("[%s] send event, type=%s", self._path, event.event_type.value)
        self._ws.send(event.model_dump_json(exclude_none=True))


def get_event_type_mapping(cls: Any) -> dict:
    res = {}
    method_list = get_methods(cls)
    for method in method_list:
        parameters = method.get("parameters", [])
        for param in parameters:
            if issubclass(param.annotation, WebsocketsEvent):
                event_type = get_model_default(param.annotation, "event_type")
                if event_type:
                    res[event_type] = method.get("function")
                    break
    return res


class WebsocketsBaseEventHandler(object):
    def on_client_error(self, cli: "WebsocketsBaseClient", e: Exception):
        log_error(f"Client Error occurred: {str(e)}")
        log_error(f"Stack trace:\n{traceback.format_exc()}")

    def on_error(self, cli: "WebsocketsBaseClient", e: Exception):
        log_error(f"Error occurred: {str(e)}")
        log_error(f"Stack trace:\n{traceback.format_exc()}")

    def on_closed(self, cli: "WebsocketsBaseClient"):
        pass

    def to_dict(self, origin: Optional[Dict[WebsocketsEventType, Callable]] = None):
        res = {
            WebsocketsEventType.CLIENT_ERROR: self.on_client_error,
            WebsocketsEventType.ERROR: self.on_error,
            WebsocketsEventType.CLOSED: self.on_closed,
        }

        res.update(get_event_type_mapping(self))
        res.update(origin or {})
        return res


class AsyncWebsocketsBaseClient(abc.ABC):
    class State(DynamicStrEnum):
        """
        initialized, connecting, connected, closing, closed
        """

        INITIALIZED = "initialized"
        CONNECTING = "connecting"
        CONNECTED = "connected"
        CLOSING = "closing"
        CLOSED = "closed"

    def __init__(
        self,
        base_url: str,
        requester: Requester,
        path: str,
        event_factory: WebsocketsEventFactory,
        query: Optional[Dict[str, str]] = None,
        on_event: Optional[Dict[WebsocketsEventType, Callable]] = None,
        wait_events: Optional[List[WebsocketsEventType]] = None,
        **kwargs,
    ):
        self._state = self.State.INITIALIZED
        self._base_url = remove_url_trailing_slash(base_url)
        self._requester = requester
        self._path = path
        self._ws_url = self._base_url + "/" + path
        if query:
            self._ws_url += "?" + "&".join([f"{k}={v}" for k, v in query.items()])
        self._on_event = on_event.copy() if on_event else {}
        self._headers = kwargs.get("headers")
        self._wait_events = wait_events.copy() if wait_events else []
        self._event_factory = event_factory

        self._input_queue: asyncio.Queue[Optional[WebsocketsEvent]] = asyncio.Queue()
        self._ws: Optional[AsyncWebsocketClientConnection] = None
        self._send_task: Optional[asyncio.Task] = None
        self._receive_task: Optional[asyncio.Task] = None

    @asynccontextmanager
    async def __call__(self):
        try:
            await self.connect()
            yield self
        finally:
            await self.close()

    async def connect(self):
        if self._state != self.State.INITIALIZED:
            raise ValueError(f"Cannot connect in {self._state.value} state")
        self._state = self.State.CONNECTING
        headers = {
            "X-Coze-Client-User-Agent": coze_client_user_agent(),
            **(self._headers or {}),
        }

        await self._requester.async_auth_header(headers)

        try:
            self._ws = await asyncio_connect(
                self._ws_url,
                user_agent_header=user_agent(),
                additional_headers=headers,
            )
            self._state = self.State.CONNECTED
            log_info("[%s] connected to websocket", self._path)

            self._send_task = asyncio.create_task(self._send_loop())
            self._receive_task = asyncio.create_task(self._receive_loop())
        except InvalidStatus as e:
            raise CozeAPIError(None, f"{e}", e.response.headers.get("x-tt-logid")) from e

    async def wait(self, events: Optional[List[WebsocketsEventType]] = None, wait_all=True) -> None:
        if events is None:
            events = self._wait_events
        await self._wait_completed(events, wait_all=wait_all)

    def on(self, event_type: WebsocketsEventType, handler: Callable):
        self._on_event[event_type] = handler

    async def close(self) -> None:
        if self._state not in (self.State.CONNECTED, self.State.CONNECTING):
            return
        self._state = self.State.CLOSING
        await self._close()
        self._state = self.State.CLOSED

    async def _send_loop(self) -> None:
        try:
            while True:
                event = await self._input_queue.get()
                await self._send_event(event)
                self._input_queue.task_done()
        except Exception as e:
            await self._handle_error(e)

    async def _receive_loop(self) -> None:
        try:
            while True:
                if not self._ws:
                    log_debug("[%s] empty websocket conn, close", self._path)
                    break

                data = await self._ws.recv()
                message = json.loads(data)
                event_type = message.get("event_type")
                _log_receive_event(self._path, event_type, data)

                handler = self._on_event.get(event_type)
                event = self._parse_event(message)
                if handler and event:
                    await handler(self, event)
        except Exception as e:
            await self._handle_error(e)

    def _parse_event(self, message: Dict) -> Optional[WebsocketsEvent]:
        event_id = message.get("id") or ""
        event_type = message.get("event_type") or ""
        detail = WebsocketsEvent.Detail.model_validate(message.get("detail") or {})
        data = message.get("data") or {}
        if event_type == WebsocketsEventType.ERROR.value:
            code, msg = data.get("code") or 0, data.get("msg") or ""
            return WebsocketsErrorEvent.model_validate(
                {
                    "id": event_id,
                    "detail": detail,
                    "data": CozeAPIError(code, msg, WebsocketsEvent.Detail.model_validate(detail).logid),
                }
            )
        return self._event_factory.create_event(self._path, message)

    async def _wait_completed(self, wait_events: List[WebsocketsEventType], wait_all: bool) -> None:
        future: asyncio.Future[None] = asyncio.Future()
        completed_events = set()

        def _wrap_handler(event_type: WebsocketsEventType, original_handler):
            async def wrapped(client, event):
                # 先执行原始处理函数
                if original_handler:
                    await original_handler(client, event)

                # 再检查完成条件
                completed_events.add(event_type)
                if wait_all:
                    # 所有事件都需要完成
                    if completed_events == set(wait_events):
                        if not future.done():
                            future.set_result(None)
                else:
                    # 任意一个事件完成即可
                    if not future.done():
                        future.set_result(None)

            return wrapped

        # 为每个指定的事件类型包装处理函数
        origin_handlers = {}
        for event_type in wait_events:
            original_handler = self._on_event.get(event_type)
            origin_handlers[event_type] = original_handler
            self._on_event[event_type] = _wrap_handler(event_type, original_handler)

        try:
            # 等待直到满足完成条件
            await future
        finally:
            # 恢复所有原来的处理函数
            for event_type in wait_events:
                original_handler = origin_handlers.get(event_type)
                if original_handler:
                    self._on_event[event_type] = original_handler

    async def _handle_error(self, error: Exception) -> None:
        handler = self._on_event.get(WebsocketsEventType.ERROR)
        if handler:
            await handler(self, error)
        else:
            raise error

    async def _close(self) -> None:
        log_info("[%s] connect closed", self._path)
        if self._send_task:
            self._send_task.cancel()
        if self._receive_task:
            self._receive_task.cancel()

        if self._ws:
            await self._ws.close()
            self._ws = None

        while not self._input_queue.empty():
            await self._input_queue.get()

        handler = self._on_event.get(WebsocketsEventType.CLOSED)
        if handler:
            await handler(self)

    async def _send_event(self, event: Optional[WebsocketsEvent] = None) -> None:
        if not event or not self._ws:
            return
        _log_send_event(self._path, event)
        await self._ws.send(event.model_dump_json(exclude_none=True))


class AsyncWebsocketsBaseEventHandler(object):
    async def on_client_error(self, cli: "WebsocketsBaseClient", e: Exception):
        log_error(f"Client Error occurred: {str(e)}")
        log_error(f"Stack trace:\n{traceback.format_exc()}")

    async def on_error(self, cli: "AsyncWebsocketsBaseClient", e: CozeAPIError):
        log_error(f"Error occurred: {str(e)}")
        log_error(f"Stack trace:\n{traceback.format_exc()}")

    async def on_closed(self, cli: "AsyncWebsocketsBaseClient"):
        pass

    def to_dict(self, origin: Optional[Dict[WebsocketsEventType, Callable]] = None):
        res = {
            WebsocketsEventType.CLIENT_ERROR: self.on_client_error,
            WebsocketsEventType.ERROR: self.on_error,
            WebsocketsEventType.CLOSED: self.on_closed,
        }

        res.update(get_event_type_mapping(self))
        res.update(origin or {})
        return res



from typing import Optional

from pydantic import Field

from cozepy.model import CozeModel, DynamicStrEnum
from cozepy.request import Requester
from cozepy.util import remove_none_values, remove_url_trailing_slash


class RoomAudioConfig(CozeModel):
    # 房间音频编码格式，支持设置为：
    # AACLC: AAC-LC 编码格式。
    # G711A: G711A 编码格式。
    # OPUS: （默认）Opus 编码格式。
    # G722: G.722 编码格式。
    codec: Optional[str]


class RoomVideoConfig(CozeModel):
    # 房间视频编码格式，支持设置为：
    # H264:（默认）H264 编码格式。
    # BYTEVC1: 火山引擎自研的视频编码格式。
    codec: Optional[str]
    # 房间视频流类型, 支持 main 和 screen。
    # main: 主流，包括通过摄像头/麦克风的内部采集机制获取的流，以及通过自定义采集方式获取的流。
    # screen: 屏幕流，用于屏幕共享或屏幕录制的视频流。
    stream_video_type: Optional[str]
    # 视频抽帧速率, 默认值是 1, 范围 [1, 24]
    video_frame_rate: Optional[int]
    # 视频帧过期时间,单位为s，默认值是1，范围[1, 10]
    video_frame_expire_duration: Optional[int]


class RoomMode(DynamicStrEnum):
    DEFAULT = "default"
    S2S = "s2s"
    PODCAST = "podcast"
    TRANSLATE = "translate"


"""
struct TranslateConfig {
    1: optional string From (go.tag = "json:\"from\"") // 翻译源语言
    2: optional string To (go.tag = "json:\"to\"") // 翻译目标语言
}
"""


class TranslateConfig(CozeModel):
    from_: Optional[str] = Field(alias="from")
    to: Optional[str]


class RoomConfig(CozeModel):
    # The audio config of the room.
    audio_config: Optional[RoomAudioConfig]
    # The video config of the room.
    video_config: Optional[RoomVideoConfig]
    # 自定义开场白
    prologue_content: Optional[str]
    # 房间模式
    room_mode: Optional[RoomMode]
    # 同传配置，仅在房间模式为同传时生效
    translate_config: Optional[TranslateConfig]
    # 在进房后等待多长时间播放开场白，默认是500ms，[0, 5000]
    prologue_delay_duration_ms: Optional[int]


class CreateRoomResp(CozeModel):
    # Token to join the room
    token: str
    # The id of user
    uid: str
    # The id of room
    room_id: str
    # App id to join the room
    app_id: str


class RoomsClient(object):
    """
    Room service client.
    """

    def __init__(self, base_url: str, requester: Requester):
        self._base_url = remove_url_trailing_slash(base_url)
        self._requester = requester

    def create(
        self,
        *,
        bot_id: str,
        voice_id: Optional[str] = None,
        conversation_id: Optional[str] = None,
        uid: Optional[str] = None,
        workflow_id: Optional[str] = None,
        config: Optional[RoomConfig] = None,
    ) -> CreateRoomResp:
        """
        create rtc room

        :param bot_id: The id of the bot.
        :param voice_id: The voice id of the voice.
        :param conversation_id: The id of the conversation.
        :param uid: The id of the user. if not provided, it will be generated by the server.
        :param workflow_id: The id of the workflow.
        :param config: The config of the room.
        :return: create room result
        """
        url = f"{self._base_url}/v1/audio/rooms"
        body = remove_none_values(
            {
                "bot_id": bot_id,
                "voice_id": voice_id,
                "conversation_id": conversation_id,
                "uid": uid,
                "workflow_id": workflow_id,
                "config": config.model_dump() if config else None,
            }
        )
        return self._requester.request("post", url, stream=False, cast=CreateRoomResp, body=body)


class AsyncRoomsClient(object):
    """
    Room service async client.
    """

    def __init__(self, base_url: str, requester: Requester):
        self._base_url = remove_url_trailing_slash(base_url)
        self._requester = requester

    async def create(
        self,
        *,
        bot_id: str,
        voice_id: Optional[str] = None,
        conversation_id: Optional[str] = None,
        uid: Optional[str] = None,
        workflow_id: Optional[str] = None,
        config: Optional[RoomConfig] = None,
    ) -> CreateRoomResp:
        """
        create rtc room

        :param bot_id: The id of the bot.
        :param voice_id: The voice id of the voice.
        :param conversation_id: The id of the conversation.
        :param uid: The id of the user. if not provided, it will be generated by the server.
        :param workflow_id: The id of the workflow.
        :param config: The config of the room.
        :return: create room result
        """
        url = f"{self._base_url}/v1/audio/rooms"
        body = remove_none_values(
            {
                "bot_id": bot_id,
                "voice_id": voice_id,
                "conversation_id": conversation_id,
                "uid": uid,
                "workflow_id": workflow_id,
                "config": config.model_dump() if config else None,
            }
        )
        return await self._requester.arequest("post", url, stream=False, cast=CreateRoomResp, body=body)



from typing import List, Optional

from cozepy.files import FileTypes, _try_fix_file
from cozepy.model import AsyncNumberPaged, CozeModel, HTTPRequest, NumberPaged, NumberPagedResponse
from cozepy.request import Requester
from cozepy.util import remove_none_values, remove_url_trailing_slash


class UserInfo(CozeModel):
    id: str
    name: str
    nickname: str
    avatar_url: str


class VoicePrintGroupFeature(CozeModel):
    id: str
    group_id: str
    name: str
    audio_url: str
    created_at: int
    updated_at: int
    desc: str
    icon_url: str
    user_info: UserInfo


class CreateVoicePrintGroupFeatureResp(CozeModel):
    id: str


class UpdateVoicePrintGroupFeatureResp(CozeModel):
    pass


class DeleteVoicePrintGroupFeatureResp(CozeModel):
    pass


class _PrivateListVoicePrintGroupFeatureData(CozeModel, NumberPagedResponse[VoicePrintGroupFeature]):
    items: List[VoicePrintGroupFeature]
    total: int

    def get_total(self) -> Optional[int]:
        return self.total

    def get_has_more(self) -> Optional[bool]:
        return None

    def get_items(self) -> List[VoicePrintGroupFeature]:
        return self.items


class VoiceprintGroupsFeaturesClient(object):
    def __init__(self, base_url: str, requester: Requester):
        self._base_url = remove_url_trailing_slash(base_url)
        self._requester = requester

    def create(
        self,
        *,
        group_id: str,
        name: str,
        file: FileTypes,
        desc: Optional[str] = None,
        sample_rate: Optional[int] = None,
        channel: Optional[int] = None,
        **kwargs,
    ) -> CreateVoicePrintGroupFeatureResp:
        url = f"{self._base_url}/v1/audio/voiceprint_groups/{group_id}/features"
        headers: Optional[dict] = kwargs.get("headers")
        files = {"file": _try_fix_file(file)}
        body = remove_none_values(
            {
                "name": name,
                "desc": desc,
                "sample_rate": sample_rate,
                "channel": channel,
            }
        )

        return self._requester.request(
            "post", url, False, cast=CreateVoicePrintGroupFeatureResp, headers=headers, body=body, files=files
        )

    def update(
        self,
        *,
        group_id: str,
        feature_id: str,
        name: Optional[str] = None,
        desc: Optional[str] = None,
        file: Optional[FileTypes] = None,
        sample_rate: Optional[int] = None,
        channel: Optional[int] = None,
        **kwargs,
    ) -> UpdateVoicePrintGroupFeatureResp:
        url = f"{self._base_url}/v1/audio/voiceprint_groups/{group_id}/features/{feature_id}"
        headers: Optional[dict] = kwargs.get("headers")
        files = {"file": _try_fix_file(file)} if file else {}
        body = remove_none_values(
            {
                "name": name,
                "desc": desc,
                "sample_rate": sample_rate,
                "channel": channel,
            }
        )

        return self._requester.request(
            "put", url, False, cast=UpdateVoicePrintGroupFeatureResp, headers=headers, body=body, files=files
        )

    def delete(
        self,
        *,
        group_id: str,
        feature_id: str,
        **kwargs,
    ) -> DeleteVoicePrintGroupFeatureResp:
        url = f"{self._base_url}/v1/audio/voiceprint_groups/{group_id}/features/{feature_id}"
        headers: Optional[dict] = kwargs.get("headers")

        return self._requester.request("delete", url, False, cast=DeleteVoicePrintGroupFeatureResp, headers=headers)

    def list(
        self,
        *,
        group_id: str,
        page_num: int = 1,
        page_size: int = 10,
    ) -> NumberPaged[VoicePrintGroupFeature]:
        url = f"{self._base_url}/v1/audio/voiceprint_groups/{group_id}/features"

        def request_maker(i_page_num: int, i_page_size: int) -> HTTPRequest:
            return self._requester.make_request(
                "GET",
                url,
                params=remove_none_values(
                    {
                        "page_num": i_page_num,
                        "page_size": i_page_size,
                    }
                ),
                cast=_PrivateListVoicePrintGroupFeatureData,
                stream=False,
            )

        return NumberPaged(
            page_num=page_num,
            page_size=page_size,
            requestor=self._requester,
            request_maker=request_maker,
        )


class AsyncVoiceprintGroupsFeaturesClient(object):
    def __init__(self, base_url: str, requester: Requester):
        self._base_url = remove_url_trailing_slash(base_url)
        self._requester = requester

    async def create(
        self,
        *,
        group_id: str,
        name: str,
        file: FileTypes,
        desc: Optional[str] = None,
        sample_rate: Optional[int] = None,
        channel: Optional[int] = None,
        **kwargs,
    ) -> CreateVoicePrintGroupFeatureResp:
        url = f"{self._base_url}/v1/audio/voiceprint_groups/{group_id}/features"
        headers: Optional[dict] = kwargs.get("headers")
        files = {"file": _try_fix_file(file)}
        body = remove_none_values(
            {
                "name": name,
                "desc": desc,
                "sample_rate": sample_rate,
                "channel": channel,
            }
        )

        return await self._requester.arequest(
            "post", url, False, cast=CreateVoicePrintGroupFeatureResp, headers=headers, body=body, files=files
        )

    async def update(
        self,
        *,
        group_id: str,
        feature_id: str,
        name: Optional[str] = None,
        desc: Optional[str] = None,
        file: Optional[FileTypes] = None,
        sample_rate: Optional[int] = None,
        channel: Optional[int] = None,
        **kwargs,
    ) -> UpdateVoicePrintGroupFeatureResp:
        url = f"{self._base_url}/v1/audio/voiceprint_groups/{group_id}/features/{feature_id}"
        headers: Optional[dict] = kwargs.get("headers")
        files = {"file": _try_fix_file(file)} if file else {}
        body = remove_none_values(
            {
                "name": name,
                "desc": desc,
                "sample_rate": sample_rate,
                "channel": channel,
            }
        )

        return await self._requester.arequest(
            "put", url, False, cast=UpdateVoicePrintGroupFeatureResp, headers=headers, body=body, files=files
        )

    async def delete(
        self,
        *,
        group_id: str,
        feature_id: str,
        **kwargs,
    ) -> DeleteVoicePrintGroupFeatureResp:
        url = f"{self._base_url}/v1/audio/voiceprint_groups/{group_id}/features/{feature_id}"
        headers: Optional[dict] = kwargs.get("headers")

        return await self._requester.arequest(
            "delete", url, False, cast=DeleteVoicePrintGroupFeatureResp, headers=headers
        )

    async def list(
        self,
        *,
        group_id: str,
        page_num: int = 1,
        page_size: int = 100,
    ) -> AsyncNumberPaged[VoicePrintGroupFeature]:
        url = f"{self._base_url}/v1/audio/voiceprint_groups/{group_id}/features"

        async def request_maker(i_page_num: int, i_page_size: int) -> HTTPRequest:
            return await self._requester.amake_request(
                "GET",
                url,
                params=remove_none_values(
                    {
                        "page_num": i_page_num,
                        "page_size": i_page_size,
                    }
                ),
                cast=_PrivateListVoicePrintGroupFeatureData,
                stream=False,
            )

        return await AsyncNumberPaged.build(
            page_num=page_num,
            page_size=page_size,
            requestor=self._requester,
            request_maker=request_maker,
        )



from typing import TYPE_CHECKING, Optional

from cozepy.request import Requester
from cozepy.util import remove_url_trailing_slash

if TYPE_CHECKING:
    from .live import AsyncLiveClient, LiveClient
    from .rooms import AsyncRoomsClient, RoomsClient
    from .speech import AsyncSpeechClient, SpeechClient
    from .transcriptions import AsyncTranscriptionsClient, TranscriptionsClient
    from .voiceprint_groups import AsyncVoiceprintGroupsClient, VoiceprintGroupsClient
    from .voices import AsyncVoicesClient, VoicesClient


class AudioClient(object):
    def __init__(self, base_url: str, requester: Requester):
        self._base_url = remove_url_trailing_slash(base_url)
        self._requester = requester

        self._rooms: Optional[RoomsClient] = None
        self._voices: Optional[VoicesClient] = None
        self._speech: Optional[SpeechClient] = None
        self._transcriptions: Optional[TranscriptionsClient] = None
        self._voiceprint_groups: Optional[VoiceprintGroupsClient] = None
        self._live: Optional[LiveClient] = None

    @property
    def rooms(self) -> "RoomsClient":
        if self._rooms is None:
            from .rooms import RoomsClient

            self._rooms = RoomsClient(base_url=self._base_url, requester=self._requester)
        return self._rooms

    @property
    def speech(self) -> "SpeechClient":
        if self._speech is None:
            from .speech import SpeechClient

            self._speech = SpeechClient(base_url=self._base_url, requester=self._requester)
        return self._speech

    @property
    def transcriptions(self) -> "TranscriptionsClient":
        if self._transcriptions is None:
            from .transcriptions import TranscriptionsClient

            self._transcriptions = TranscriptionsClient(base_url=self._base_url, requester=self._requester)
        return self._transcriptions

    @property
    def voices(self) -> "VoicesClient":
        if self._voices is None:
            from .voices import VoicesClient

            self._voices = VoicesClient(base_url=self._base_url, requester=self._requester)
        return self._voices

    @property
    def voiceprint_groups(self) -> "VoiceprintGroupsClient":
        if self._voiceprint_groups is None:
            from .voiceprint_groups import VoiceprintGroupsClient

            self._voiceprint_groups = VoiceprintGroupsClient(base_url=self._base_url, requester=self._requester)
        return self._voiceprint_groups

    @property
    def live(self) -> "LiveClient":
        if self._live is None:
            from .live import LiveClient

            self._live = LiveClient(base_url=self._base_url, requester=self._requester)
        return self._live


class AsyncAudioClient(object):
    def __init__(self, base_url: str, requester: Requester):
        self._base_url = remove_url_trailing_slash(base_url)
        self._requester = requester

        self._rooms: Optional[AsyncRoomsClient] = None
        self._voices: Optional[AsyncVoicesClient] = None
        self._speech: Optional[AsyncSpeechClient] = None
        self._transcriptions: Optional[AsyncTranscriptionsClient] = None
        self._voiceprint_groups: Optional[AsyncVoiceprintGroupsClient] = None
        self._live: Optional[AsyncLiveClient] = None

    @property
    def rooms(self) -> "AsyncRoomsClient":
        if self._rooms is None:
            from .rooms import AsyncRoomsClient

            self._rooms = AsyncRoomsClient(base_url=self._base_url, requester=self._requester)
        return self._rooms

    @property
    def speech(self) -> "AsyncSpeechClient":
        if self._speech is None:
            from .speech import AsyncSpeechClient

            self._speech = AsyncSpeechClient(base_url=self._base_url, requester=self._requester)
        return self._speech

    @property
    def voices(self) -> "AsyncVoicesClient":
        if self._voices is None:
            from .voices import AsyncVoicesClient

            self._voices = AsyncVoicesClient(base_url=self._base_url, requester=self._requester)
        return self._voices

    @property
    def transcriptions(self) -> "AsyncTranscriptionsClient":
        if self._transcriptions is None:
            from .transcriptions import AsyncTranscriptionsClient

            self._transcriptions = AsyncTranscriptionsClient(base_url=self._base_url, requester=self._requester)
        return self._transcriptions

    @property
    def voiceprint_groups(self) -> "AsyncVoiceprintGroupsClient":
        if self._voiceprint_groups is None:
            from .voiceprint_groups import AsyncVoiceprintGroupsClient

            self._voiceprint_groups = AsyncVoiceprintGroupsClient(base_url=self._base_url, requester=self._requester)
        return self._voiceprint_groups

    @property
    def live(self) -> "AsyncLiveClient":
        if self._live is None:
            from .live import AsyncLiveClient

            self._live = AsyncLiveClient(base_url=self._base_url, requester=self._requester)
        return self._live
